@@ -1,4 +1,4 @@
-/**
+/*
  * XML Security Library (http://www.aleksey.com/xmlsec).
  *
  * List of pointers.
@@ -6,18 +6,19 @@
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  *
- * Copyright (C) 2002-2003 Aleksey Sanin <aleksey@aleksey.com>
+ * Copyright (C) 2002-2022 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  */
 #ifndef __XMLSEC_LIST_H__
 #define __XMLSEC_LIST_H__
 
+#include <xmlsec/exports.h>
+#include <xmlsec/xmlsec.h>
+#include <xmlsec/buffer.h>
+
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
 
-#include <xmlsec/xmlsec.h>
-#include <xmlsec/buffer.h>
-
 typedef const struct _xmlSecPtrListKlass                        xmlSecPtrListKlass,
                                                                 *xmlSecPtrListId;
 typedef struct _xmlSecPtrList                                   xmlSecPtrList,
@@ -147,7 +148,7 @@ typedef void                    (*xmlSecPtrDebugDumpItemMethod) (xmlSecPtr ptr,
 /**
  * xmlSecPtrListKlass:
  * @name:               the list klass name.
- * @duplicateItem:      the duplciate item method.
+ * @duplicateItem:      the duplicate item method.
  * @destroyItem:        the destroy item method.
  * @debugDumpItem:      the debug dump item method.
  * @debugXmlDumpItem:   the debug dump item in xml format method.
