@@ -6,13 +6,14 @@
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  *
- * Copyright (C) 2002-2016 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
+ * Copyright (C) 2002-2022 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  */
 #ifndef __XMLSEC_PARSER_H__
 #define __XMLSEC_PARSER_H__
 
 #include <libxml/tree.h>
 
+#include <xmlsec/exports.h>
 #include <xmlsec/xmlsec.h>
 #include <xmlsec/transforms.h>
 
@@ -30,7 +31,11 @@ XMLSEC_EXPORT xmlDocPtr         xmlSecParseMemoryExt    (const xmlSecByte *prefi
                                                          xmlSecSize bufferSize,
                                                          const xmlSecByte *postfix,
                                                          xmlSecSize postfixSize);
-XMLSEC_EXPORT void				xmlSecParsePrepareCtxt	(xmlParserCtxtPtr ctxt);
+XMLSEC_EXPORT void              xmlSecParsePrepareCtxt  (xmlParserCtxtPtr ctxt);
+
+XMLSEC_EXPORT int               xmlSecParserGetDefaultOptions(void);
+XMLSEC_EXPORT void              xmlSecParserSetDefaultOptions(int options);
+
 
 /**
  * xmlSecTransformXmlParserId:
