@@ -1,12 +1,17 @@
-/**
+/*
  * XML Security Library (http://www.aleksey.com/xmlsec).
  *
- * Enveloped transform.
  *
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  *
- * Copyright (C) 2002-2003 Aleksey Sanin <aleksey@aleksey.com>
+ * Copyright (C) 2002-2022 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
+ */
+/**
+ * SECTION:enveloped
+ * @Short_description: Enveloped transform implementation.
+ * @Stability: Private
+ *
  */
 #include "globals.h"
 
@@ -73,10 +78,11 @@ static xmlSecTransformKlass xmlSecTransformEnvelopedKlass = {
  * result from replacing T with an XPath transform containing the following
  * XPath parameter element:
  *
- * <XPath xmlns:dsig="&dsig;">
- *   count(ancestor-or-self::dsig:Signature |
- *   here()/ancestor::dsig:Signature[1]) >
- *   count(ancestor-or-self::dsig:Signature)</XPath>
+ *   <XPath xmlns:dsig="...">
+ *     count(ancestor-or-self::dsig:Signature |
+ *     here()/ancestor::dsig:Signature[1]) >
+ *     count(ancestor-or-self::dsig:Signature)
+ *   </XPath>
  *
  * The input and output requirements of this transform are identical to
  * those of the XPath transform, but may only be applied to a node-set from
@@ -105,44 +111,35 @@ xmlSecTransformEnvelopedExecute(xmlSecTransformPtr transform, int last,
     xmlSecAssert2(transformCtx != NULL, -1);
 
     if((transform->inNodes != NULL) && (transform->inNodes->doc != transform->hereNode->doc)) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    xmlSecErrorsSafeString(xmlSecTransformGetName(transform)),
-                    NULL,
-                    XMLSEC_ERRORS_R_TRANSFORM_SAME_DOCUMENT_REQUIRED,
-                    XMLSEC_ERRORS_NO_MESSAGE);
+        xmlSecOtherError(XMLSEC_ERRORS_R_TRANSFORM_SAME_DOCUMENT_REQUIRED,
+                         xmlSecTransformGetName(transform),
+                         NULL);
         return(-1);
     }
 
     /* find signature node and get all its children in the nodes set */
     node = xmlSecFindParent(transform->hereNode, xmlSecNodeSignature, xmlSecDSigNs);
     if(node == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    xmlSecErrorsSafeString(xmlSecTransformGetName(transform)),
-                    xmlSecErrorsSafeString(xmlSecNodeSignature),
-                    XMLSEC_ERRORS_R_NODE_NOT_FOUND,
-                    XMLSEC_ERRORS_NO_MESSAGE);
+        xmlSecNodeNotFoundError("xmlSecFindParent", transform->hereNode,
+                                xmlSecNodeSignature,
+                                xmlSecTransformGetName(transform));
         return(-1);
     }
 
     children = xmlSecNodeSetGetChildren(node->doc, node, 1, 1);
     if(children == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    xmlSecErrorsSafeString(xmlSecTransformGetName(transform)),
-                    "xmlSecNodeSetGetChildren",
-                    XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                    "node=%s",
-                    xmlSecErrorsSafeString(xmlSecNodeGetName(node)));
+        xmlSecInternalError2("xmlSecNodeSetGetChildren",
+                             xmlSecTransformGetName(transform),
+                             "node=%s",
+                             xmlSecErrorsSafeString(xmlSecNodeGetName(node)));
         return(-1);
     }
 
     /* intersect <dsig:Signature/> node children with input nodes (if exist) */
     transform->outNodes = xmlSecNodeSetAdd(transform->inNodes, children, xmlSecNodeSetIntersection);
     if(transform->outNodes == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    xmlSecErrorsSafeString(xmlSecTransformGetName(transform)),
-                    "xmlSecNodeSetAdd",
-                    XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                    XMLSEC_ERRORS_NO_MESSAGE);
+        xmlSecInternalError("xmlSecNodeSetAdd",
+                            xmlSecTransformGetName(transform));
         xmlSecNodeSetDestroy(children);
         return(-1);
     }
