@@ -1,11 +1,19 @@
-/**
- * XMLSec library
+/*
+ * XML Security Library (http://www.aleksey.com/xmlsec).
+ *
  *
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  *
- * Copyright (C) 2002-2016 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
+ * Copyright (C) 2002-2022 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  */
+/**
+ * SECTION:asn1
+ * @Short_description: ASN1 support functions for GCrypt.
+ * @Stability: Private
+ *
+ */
+
 #include "globals.h"
 
 #include <string.h>
@@ -19,6 +27,7 @@
 #include <xmlsec/gcrypt/crypto.h>
 
 #include "asn1.h"
+#include "../cast_helpers.h"
 
 /**************************************************************************
  *
@@ -87,12 +96,12 @@ struct tag_info
    that the encoded length does not exhaust the length of the provided
    buffer. */
 static int
-xmlSecGCryptAsn1ParseTag (xmlSecByte const **buffer, xmlSecSize *buflen, struct tag_info *ti)
+xmlSecGCryptAsn1ParseTag (xmlSecByte const **buffer, unsigned long *buflen, struct tag_info *ti)
 {
-    int c;
+    unsigned long c;
     unsigned long tag;
     const xmlSecByte *buf;
-    xmlSecSize length;
+    unsigned long length;
 
     xmlSecAssert2(buffer != NULL, -1);
     xmlSecAssert2((*buffer) != NULL, -1);
@@ -111,7 +120,7 @@ xmlSecGCryptAsn1ParseTag (xmlSecByte const **buffer, xmlSecSize *buflen, struct
     if (length <= 0) {
         return(-1); /* Premature EOF.  */
     }
-    c = *buf++; 
+    c = *buf++;
     length--;
     ti->nhdr++;
 
@@ -126,7 +135,7 @@ xmlSecGCryptAsn1ParseTag (xmlSecByte const **buffer, xmlSecSize *buflen, struct
             if (length <= 0) {
                 return(-1); /* Premature EOF.  */
             }
-            c = *buf++; 
+            c = *buf++;
             length--;
             ti->nhdr++;
             tag |= (c & 0x7f);
@@ -138,7 +147,7 @@ xmlSecGCryptAsn1ParseTag (xmlSecByte const **buffer, xmlSecSize *buflen, struct
     if(length <= 0) {
         return -1; /* Premature EOF. */
     }
-    c = *buf++; 
+    c = *buf++;
     length--;
     ti->nhdr++;
 
@@ -149,7 +158,7 @@ xmlSecGCryptAsn1ParseTag (xmlSecByte const **buffer, xmlSecSize *buflen, struct
     } else if (c == 0xff) {
         return -1; /* Forbidden length value.  */
     } else {
-        xmlSecSize len = 0;
+        unsigned long len = 0;
         int count = c & 0x7f;
 
         for (; count; count--) {
@@ -179,10 +188,10 @@ xmlSecGCryptAsn1ParseTag (xmlSecByte const **buffer, xmlSecSize *buflen, struct
 }
 
 static int
-xmlSecGCryptAsn1ParseIntegerSequence(xmlSecByte const **buffer, xmlSecSize *buflen,
+xmlSecGCryptAsn1ParseIntegerSequence(xmlSecByte const **buffer, xmlSecSize* buflen,
                                      gcry_mpi_t * params, int params_size) {
     const xmlSecByte *buf;
-    xmlSecSize length;
+    unsigned long length;
     struct tag_info ti;
     gcry_error_t err;
     int idx = 0;
@@ -196,18 +205,14 @@ xmlSecGCryptAsn1ParseIntegerSequence(xmlSecByte const **buffer, xmlSecSize *bufl
 
     /* initialize */
     buf = *buffer;
-    length = *buflen;
+    XMLSEC_SAFE_CAST_SIZE_TO_ULONG((*buflen), length, return(-1), NULL);
 
     /* read SEQUENCE */
     memset(&ti, 0, sizeof(ti));
     ret = xmlSecGCryptAsn1ParseTag (&buf, &length, &ti);
     if((ret != 0)  || (ti.tag != TAG_SEQUENCE) || ti.class || !ti.cons || ti.ndef) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlSecGCryptAsn1ParseTag",
-                    XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                    "TAG_SEQUENCE is expected: tag=%d",
-                    (int)ti.tag);
+        xmlSecInternalError2("xmlSecGCryptAsn1ParseTag", NULL,
+            "TAG_SEQUENCE is expected: tag=%lu", ti.tag);
         return(-1);
     }
 
@@ -217,22 +222,14 @@ xmlSecGCryptAsn1ParseIntegerSequence(xmlSecByte const **buffer, xmlSecSize *bufl
         ret = xmlSecGCryptAsn1ParseTag (&buf, &length, &ti);
         if((ret != 0) || (ti.tag != TAG_INTEGER) || ti.class || ti.cons || ti.ndef)
         {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        "xmlSecGCryptAsn1ParseTag",
-                        XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                        "TAG_INTEGER is expected - index=%d, tag=%d",
-                        (int)idx, (int)ti.tag);
+            xmlSecInternalError3("xmlSecGCryptAsn1ParseTag", NULL,
+                "TAG_INTEGER is expected - index=%d, tag=%lu", idx, ti.tag);
             return(-1);
         }
 
         err = gcry_mpi_scan(&(params[idx]), GCRYMPI_FMT_USG, buf, ti.length, NULL);
         if((err != GPG_ERR_NO_ERROR) || (params[idx] == NULL)) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        "gcry_mpi_scan",
-                        XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                        XMLSEC_GCRYPT_REPORT_ERROR(err));
+            xmlSecGCryptError("gcry_mpi_scan", err, NULL);
             return(-1);
         }
         buf += ti.length;
@@ -241,18 +238,14 @@ xmlSecGCryptAsn1ParseIntegerSequence(xmlSecByte const **buffer, xmlSecSize *bufl
 
     /* did we parse everything? */
     if(length > 0) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlSecGCryptAsn1ParseTag",
-                    XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                    "too many params - cur=%d, expected=%d",
-                    (int)(idx - 1), (int)params_size);
+        xmlSecInternalError3("xmlSecGCryptAsn1ParseTag", NULL,
+            "too many params - cur=%d, expected=%d", (idx - 1), params_size);
         return(-1);
     }
 
     /* done */
     *buffer = buf;
-    *buflen = length;
+    XMLSEC_SAFE_CAST_ULONG_TO_SIZE(length, (*buflen), return(-1), NULL);
     return(idx);
 }
 
@@ -264,7 +257,7 @@ xmlSecGCryptParseDer(const xmlSecByte * der, xmlSecSize derlen,
     gcry_sexp_t s_priv_key = NULL;
     gcry_error_t err;
     gcry_mpi_t keyparms[20];
-    int keyparms_num;
+    xmlSecSize keyparms_num;
     unsigned int idx;
     int ret;
 
@@ -278,67 +271,52 @@ xmlSecGCryptParseDer(const xmlSecByte * der, xmlSecSize derlen,
         keyparms,  sizeof(keyparms) / sizeof(keyparms[0])
     );
     if(ret < 0) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlSecGCryptAsn1ParseIntegerSequence",
-                    XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                    XMLSEC_ERRORS_NO_MESSAGE);
+        xmlSecInternalError("xmlSecGCryptAsn1ParseIntegerSequence", NULL);
         goto done;
     }
-    keyparms_num = ret;
+    XMLSEC_SAFE_CAST_INT_TO_SIZE(ret, keyparms_num, goto done, NULL);
 
     /* The value of the first integer should be 0. */
     if ((keyparms_num < 1) || (gcry_mpi_cmp_ui(keyparms[0], 0) != 0)) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlSecGCryptAsn1ParseTag",
-                    XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                    "num=%d",
-                    (int)keyparms_num);
+        xmlSecInternalError2("xmlSecGCryptAsn1ParseTag", NULL,
+            "num=" XMLSEC_SIZE_FMT, keyparms_num);
         goto done;
     }
 
     /* do we need to guess the key type? not robust but the best we can do */
     if(type == xmlSecGCryptDerKeyTypeAuto) {
         switch(keyparms_num) {
-        case 3:
+        case 3U:
             /* Public RSA */
             type = xmlSecGCryptDerKeyTypePublicRsa;
             break;
-        case 5:
+        case 5U:
             /* Public DSA */
             type = xmlSecGCryptDerKeyTypePublicDsa;
             break;
-        case 6:
+        case 6U:
             /* Private DSA */
             type = xmlSecGCryptDerKeyTypePrivateDsa;
             break;
-        case 9:
+        case 9U:
             /* Private RSA */
             type = xmlSecGCryptDerKeyTypePrivateRsa;
             break;
         default:
             /* unknown */
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        "Unexpected number of parameters, unknown key type",
-                        XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                        "keyparms_num=%d", (int)keyparms_num);
+            xmlSecInvalidSizeDataError("keyparms_num", keyparms_num,
+                "the number of parameters matching key type", NULL);
             goto done;
         }
     }
 
-
     switch(type) {
 #ifndef XMLSEC_NO_DSA
     case xmlSecGCryptDerKeyTypePrivateDsa:
         /* check we have enough params */
-        if(keyparms_num != 6) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        "Private DSA key: 6 parameters exepcted",
-                        XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                        "parms_num=%d", (int)keyparms_num);
+        if(keyparms_num != 6U) {
+            xmlSecInvalidSizeError("Private DSA key params",
+                keyparms_num, (xmlSecSize)6U, NULL);
             goto done;
         }
 
@@ -354,11 +332,7 @@ xmlSecGCryptParseDer(const xmlSecByte * der, xmlSecSize derlen,
                 keyparms[1], keyparms[2], keyparms[3], keyparms[4], keyparms[5]
         );
         if((err != GPG_ERR_NO_ERROR) || (s_priv_key == NULL)) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        "gcry_sexp_build(private-key/dsa)",
-                        XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                        XMLSEC_GCRYPT_REPORT_ERROR(err));
+            xmlSecGCryptError("gcry_sexp_build(private-key/dsa)", err, NULL);
             goto done;
         }
 
@@ -367,32 +341,20 @@ xmlSecGCryptParseDer(const xmlSecByte * der, xmlSecSize derlen,
                 keyparms[1], keyparms[2], keyparms[3], keyparms[5]
         );
         if((err != GPG_ERR_NO_ERROR) || (s_pub_key == NULL)) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        "gcry_sexp_build(public-key/dsa)",
-                        XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                        XMLSEC_GCRYPT_REPORT_ERROR(err));
+            xmlSecGCryptError("gcry_sexp_build(public-key/dsa)", err, NULL);
             goto done;
         }
 
         /* construct key and key data */
         key_data = xmlSecKeyDataCreate(xmlSecGCryptKeyDataDsaId);
         if(key_data == NULL) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        "xmlSecKeyDataCreate",
-                        XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                        "xmlSecGCryptKeyDataDsaId");
+            xmlSecInternalError("xmlSecKeyDataCreate(xmlSecGCryptKeyDataDsaId)", NULL);
             goto done;
         }
 
         ret = xmlSecGCryptKeyDataDsaAdoptKeyPair(key_data, s_pub_key, s_priv_key);
         if(ret < 0) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        "xmlSecGCryptKeyDataDsaAdoptKey",
-                        XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                        "xmlSecGCryptKeyDataDsaId");
+            xmlSecInternalError("xmlSecGCryptKeyDataDsaAdoptKey(xmlSecGCryptKeyDataDsaId)", NULL);
             xmlSecKeyDataDestroy(key_data);
             key_data = NULL;
             goto done;
@@ -403,12 +365,9 @@ xmlSecGCryptParseDer(const xmlSecByte * der, xmlSecSize derlen,
 
     case xmlSecGCryptDerKeyTypePublicDsa:
         /* check we have enough params */
-        if(keyparms_num != 5) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        "Public DSA key: 5 parameters exepcted",
-                        XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                        "parms_num=%d", (int)keyparms_num);
+        if(keyparms_num != 5U) {
+            xmlSecInvalidSizeError("Public DSA key params",
+                keyparms_num, (xmlSecSize)5U, NULL);
             goto done;
         }
 
@@ -418,32 +377,20 @@ xmlSecGCryptParseDer(const xmlSecByte * der, xmlSecSize derlen,
                 keyparms[2], keyparms[3], keyparms[4], keyparms[1]
         );
         if((err != GPG_ERR_NO_ERROR) || (s_pub_key == NULL)) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        "gcry_sexp_build(public-key/dsa)",
-                        XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                        XMLSEC_GCRYPT_REPORT_ERROR(err));
+            xmlSecGCryptError("gcry_sexp_build(public-key/dsa)", err, NULL);
             goto done;
         }
 
         /* construct key and key data */
         key_data = xmlSecKeyDataCreate(xmlSecGCryptKeyDataDsaId);
         if(key_data == NULL) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        "xmlSecKeyDataCreate",
-                        XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                        "xmlSecGCryptKeyDataDsaId");
+            xmlSecInternalError("xmlSecKeyDataCreate(xmlSecGCryptKeyDataDsaId)", NULL);
             goto done;
         }
 
         ret = xmlSecGCryptKeyDataDsaAdoptKeyPair(key_data, s_pub_key, NULL);
         if(ret < 0) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        "xmlSecGCryptKeyDataDsaAdoptKey",
-                        XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                        "xmlSecGCryptKeyDataDsaId");
+            xmlSecInternalError("xmlSecGCryptKeyDataDsaAdoptKey(xmlSecGCryptKeyDataDsaId)", NULL);
             xmlSecKeyDataDestroy(key_data);
             key_data = NULL;
             goto done;
@@ -455,18 +402,15 @@ xmlSecGCryptParseDer(const xmlSecByte * der, xmlSecSize derlen,
 #ifndef XMLSEC_NO_RSA
     case xmlSecGCryptDerKeyTypePrivateRsa:
         /* check we have enough params */
-        if(keyparms_num != 9) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        "Private RSA key: 9 parameters exepcted",
-                        XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                        "parms_num=%d", (int)keyparms_num);
+        if(keyparms_num != 9U) {
+            xmlSecInvalidSizeError("Private RSA key params",
+                (xmlSecSize)keyparms_num, (xmlSecSize)9U, NULL);
             goto done;
         }
 
         /* Convert from OpenSSL parameter ordering to the OpenPGP order. */
         /* (http://gnupg.10057.n7.nabble.com/RSA-PKCS-1-signing-differs-from-OpenSSL-s-td27920.html) */
-        /* First check that p < q; if not swap p and q and recompute u.  */ 
+        /* First check that p < q; if not swap p and q and recompute u.  */
         if (gcry_mpi_cmp (keyparms[4], keyparms[5]) > 0) {
             gcry_mpi_swap (keyparms[4], keyparms[5]);
             gcry_mpi_invm (keyparms[8], keyparms[4], keyparms[5]);
@@ -479,11 +423,7 @@ xmlSecGCryptParseDer(const xmlSecByte * der, xmlSecSize derlen,
                          keyparms[4], keyparms[5], keyparms[8]
         );
         if((err != GPG_ERR_NO_ERROR) || (s_priv_key == NULL)) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        "gcry_sexp_build(private-key/rsa)",
-                        XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                        XMLSEC_GCRYPT_REPORT_ERROR(err));
+            xmlSecGCryptError("gcry_sexp_build(private-key/rsa)", err, NULL);
             goto done;
         }
 
@@ -492,32 +432,20 @@ xmlSecGCryptParseDer(const xmlSecByte * der, xmlSecSize derlen,
                          keyparms[1], keyparms[2]
         );
         if((err != GPG_ERR_NO_ERROR) || (s_pub_key == NULL)) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        "gcry_sexp_build(public-key/rsa)",
-                        XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                        XMLSEC_GCRYPT_REPORT_ERROR(err));
+            xmlSecGCryptError("gcry_sexp_build(public-key/rsa)", err, NULL);
             goto done;
         }
 
         /* construct key and key data */
         key_data = xmlSecKeyDataCreate(xmlSecGCryptKeyDataRsaId);
         if(key_data == NULL) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        "xmlSecKeyDataCreate",
-                        XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                        "xmlSecGCryptKeyDataRsaId");
+            xmlSecInternalError("xmlSecKeyDataCreate(xmlSecGCryptKeyDataRsaId)", NULL);
             goto done;
         }
 
         ret = xmlSecGCryptKeyDataRsaAdoptKeyPair(key_data, s_pub_key, s_priv_key);
         if(ret < 0) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        "xmlSecGCryptKeyDataRsaAdoptKey",
-                        XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                        "xmlSecGCryptKeyDataRsaId");
+            xmlSecInternalError("xmlSecGCryptKeyDataRsaAdoptKey(xmlSecGCryptKeyDataRsaId)", NULL);
             xmlSecKeyDataDestroy(key_data);
             key_data = NULL;
             goto done;
@@ -528,12 +456,9 @@ xmlSecGCryptParseDer(const xmlSecByte * der, xmlSecSize derlen,
 
     case xmlSecGCryptDerKeyTypePublicRsa:
         /* check we have enough params */
-        if(keyparms_num != 3) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        "Public RSA key: 3 parameters exepcted",
-                        XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                        "parms_num=%d", (int)keyparms_num);
+        if(keyparms_num != 3U) {
+            xmlSecInvalidSizeError("Public RSA key params",
+                keyparms_num, (xmlSecSize)3U, NULL);
             goto done;
         }
 
@@ -543,32 +468,20 @@ xmlSecGCryptParseDer(const xmlSecByte * der, xmlSecSize derlen,
                          keyparms[1], keyparms[2]
         );
         if((err != GPG_ERR_NO_ERROR) || (s_pub_key == NULL)) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        "gcry_sexp_build(public-key/rsa)",
-                        XMLSEC_ERRORS_R_CRYPTO_FAILED,
-                        XMLSEC_GCRYPT_REPORT_ERROR(err));
+            xmlSecGCryptError("gcry_sexp_build(public-key/rsa)", err, NULL);
             goto done;
         }
 
         /* construct key and key data */
         key_data = xmlSecKeyDataCreate(xmlSecGCryptKeyDataRsaId);
         if(key_data == NULL) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        "xmlSecKeyDataCreate",
-                        XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                        "xmlSecGCryptKeyDataRsaId");
+            xmlSecInternalError("xmlSecKeyDataCreate(xmlSecGCryptKeyDataRsaId)", NULL);
             goto done;
         }
 
         ret = xmlSecGCryptKeyDataRsaAdoptKeyPair(key_data, s_pub_key, NULL);
         if(ret < 0) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        "xmlSecGCryptKeyDataRsaAdoptKey",
-                        XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                        "xmlSecGCryptKeyDataRsaId");
+            xmlSecInternalError("xmlSecGCryptKeyDataRsaAdoptKey(xmlSecGCryptKeyDataRsaId)", NULL);
             xmlSecKeyDataDestroy(key_data);
             key_data = NULL;
             goto done;
@@ -578,11 +491,7 @@ xmlSecGCryptParseDer(const xmlSecByte * der, xmlSecSize derlen,
 #endif /* XMLSEC_NO_RSA */
 
     default:
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "Unsupported key type",
-                    XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                    "type=%d", (int)type);
+        xmlSecUnsupportedEnumValueError("key_type", type, NULL);
         goto done;
         break;
     }
