@@ -1,13 +1,19 @@
 /*
  * XML Security Library (http://www.aleksey.com/xmlsec).
  *
- * AES Algorithm support
  *
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  *
- * Copyright (C) 2010-2016 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
+ * Copyright (C) 2002-2022 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  */
+/**
+ * SECTION:kw_aes
+ * @Short_description: AES Key Transport transforms implementation for GCrypt.
+ * @Stability: Private
+ *
+ */
+
 #ifndef XMLSEC_NO_AES
 #include "globals.h"
 
@@ -18,31 +24,33 @@
 #include <gcrypt.h>
 
 #include <xmlsec/xmlsec.h>
-#include <xmlsec/xmltree.h>
 #include <xmlsec/keys.h>
 #include <xmlsec/transforms.h>
 #include <xmlsec/errors.h>
+#include <xmlsec/private.h>
 
 #include <xmlsec/gcrypt/crypto.h>
 
 #include "../kw_aes_des.h"
-
+#include "../cast_helpers.h"
 
 /*********************************************************************
  *
  * AES KW implementation
  *
  *********************************************************************/
-static int        xmlSecGCryptKWAesBlockEncrypt                 (const xmlSecByte * in, 
+static int        xmlSecGCryptKWAesBlockEncrypt                 (xmlSecTransformPtr transform,
+                                                                 const xmlSecByte * in,
                                                                  xmlSecSize inSize,
-                                                                 xmlSecByte * out, 
+                                                                 xmlSecByte * out,
                                                                  xmlSecSize outSize,
-                                                                 void * context);
-static int        xmlSecGCryptKWAesBlockDecrypt                 (const xmlSecByte * in, 
+                                                                 xmlSecSize * outWritten);
+static int        xmlSecGCryptKWAesBlockDecrypt                 (xmlSecTransformPtr transform,
+                                                                 const xmlSecByte * in,
                                                                  xmlSecSize inSize,
-                                                                 xmlSecByte * out, 
+                                                                 xmlSecByte * out,
                                                                  xmlSecSize outSize,
-                                                                 void * context);
+                                                                 xmlSecSize * outWritten);
 static xmlSecKWAesKlass xmlSecGCryptKWAesKlass = {
     /* callbacks */
     xmlSecGCryptKWAesBlockEncrypt,          /* xmlSecKWAesBlockEncryptMethod       encrypt; */
@@ -56,24 +64,30 @@ static xmlSecKWAesKlass xmlSecGCryptKWAesKlass = {
 
 /*********************************************************************
  *
- * AES KW transforms
+ * AES KW transform context
  *
  ********************************************************************/
 typedef struct _xmlSecGCryptKWAesCtx              xmlSecGCryptKWAesCtx,
                                                   *xmlSecGCryptKWAesCtxPtr;
 struct _xmlSecGCryptKWAesCtx {
+    xmlSecTransformKWAesCtx parentCtx;
+
     int                 cipher;
     int                 mode;
-    int                 flags;
+    unsigned int        flags;
     xmlSecSize          blockSize;
-    xmlSecSize          keyExpectedSize;
-
-    xmlSecBuffer        keyBuffer;
 };
-#define xmlSecGCryptKWAesSize     \
-    (sizeof(xmlSecTransform) + sizeof(xmlSecGCryptKWAesCtx))
-#define xmlSecGCryptKWAesGetCtx(transform) \
-    ((xmlSecGCryptKWAesCtxPtr)(((xmlSecByte*)(transform)) + sizeof(xmlSecTransform)))
+
+/******************************************************************************
+ *
+ * AES KW transforms
+ *
+ * xmlSecTransform + xmlSecGCryptKWAesCtx
+ *
+ *****************************************************************************/
+XMLSEC_TRANSFORM_DECLARE(GCryptKWAes, xmlSecGCryptKWAesCtx)
+#define xmlSecGCryptKWAesSize XMLSEC_TRANSFORM_SIZE(GCryptKWAes)
+
 #define xmlSecGCryptKWAesCheckId(transform) \
     (xmlSecTransformCheckId((transform), xmlSecGCryptTransformKWAes128Id) || \
      xmlSecTransformCheckId((transform), xmlSecGCryptTransformKWAes192Id) || \
@@ -92,6 +106,8 @@ static int      xmlSecGCryptKWAesExecute                        (xmlSecTransform
 static int
 xmlSecGCryptKWAesInitialize(xmlSecTransformPtr transform) {
     xmlSecGCryptKWAesCtxPtr ctx;
+    xmlSecSize keyExpectedSize;
+    size_t blockSize;
     int ret;
 
     xmlSecAssert2(xmlSecGCryptKWAesCheckId(transform), -1);
@@ -99,32 +115,42 @@ xmlSecGCryptKWAesInitialize(xmlSecTransformPtr transform) {
 
     ctx = xmlSecGCryptKWAesGetCtx(transform);
     xmlSecAssert2(ctx != NULL, -1);
+    memset(ctx, 0, sizeof(xmlSecGCryptKWAesCtx));
 
     if(xmlSecTransformCheckId(transform, xmlSecGCryptTransformKWAes128Id)) {
-        ctx->cipher             = GCRY_CIPHER_AES128;
-        ctx->keyExpectedSize    = XMLSEC_KW_AES128_KEY_SIZE;
+        ctx->cipher     = GCRY_CIPHER_AES128;
+        keyExpectedSize = XMLSEC_KW_AES128_KEY_SIZE;
     } else if(xmlSecTransformCheckId(transform, xmlSecGCryptTransformKWAes192Id)) {
-        ctx->cipher             = GCRY_CIPHER_AES192;
-        ctx->keyExpectedSize    = XMLSEC_KW_AES192_KEY_SIZE;
+        ctx->cipher     = GCRY_CIPHER_AES192;
+        keyExpectedSize = XMLSEC_KW_AES192_KEY_SIZE;
     } else if(xmlSecTransformCheckId(transform, xmlSecGCryptTransformKWAes256Id)) {
-        ctx->cipher             = GCRY_CIPHER_AES256;
-        ctx->keyExpectedSize    = XMLSEC_KW_AES256_KEY_SIZE;
+        ctx->cipher     = GCRY_CIPHER_AES256;
+        keyExpectedSize = XMLSEC_KW_AES256_KEY_SIZE;
     } else {
         xmlSecInvalidTransfromError(transform)
         return(-1);
     }
-    ctx->mode           = GCRY_CIPHER_MODE_CBC;
-    ctx->flags          = GCRY_CIPHER_SECURE; /* we are paranoid */
-    ctx->blockSize      = gcry_cipher_get_algo_blklen(ctx->cipher);
-    xmlSecAssert2(ctx->blockSize > 0, -1);
 
-    ret = xmlSecBufferInitialize(&(ctx->keyBuffer), 0);
+    ret = xmlSecTransformKWAesInitialize(transform, &(ctx->parentCtx),
+        &xmlSecGCryptKWAesKlass, xmlSecGCryptKeyDataAesId,
+        keyExpectedSize);
     if(ret < 0) {
-        xmlSecInternalError("xmlSecGCryptKWAesGetKey",
-                            xmlSecTransformGetName(transform));
+        xmlSecInternalError("xmlSecTransformKWAesInitialize", xmlSecTransformGetName(transform));
+        xmlSecGCryptKWAesFinalize(transform);
         return(-1);
     }
 
+    blockSize = gcry_cipher_get_algo_blklen(ctx->cipher);
+    if(blockSize <= 0) {
+        xmlSecGCryptError("gcry_cipher_get_algo_blklen", (gcry_error_t)GPG_ERR_NO_ERROR, NULL);
+        xmlSecGCryptKWAesFinalize(transform);
+        return(-1);
+    }
+
+    ctx->mode           = GCRY_CIPHER_MODE_CBC;
+    ctx->flags          = GCRY_CIPHER_SECURE; /* we are paranoid */
+    XMLSEC_SAFE_CAST_SIZE_T_TO_SIZE(blockSize, ctx->blockSize, return(-1), NULL);
+
     return(0);
 }
 
@@ -138,171 +164,65 @@ xmlSecGCryptKWAesFinalize(xmlSecTransformPtr transform) {
     ctx = xmlSecGCryptKWAesGetCtx(transform);
     xmlSecAssert(ctx != NULL);
 
-    xmlSecBufferFinalize(&(ctx->keyBuffer));
+    xmlSecTransformKWAesFinalize(transform, &(ctx->parentCtx));
+    memset(ctx, 0, sizeof(xmlSecGCryptKWAesCtx));
 }
 
 static int
 xmlSecGCryptKWAesSetKeyReq(xmlSecTransformPtr transform,  xmlSecKeyReqPtr keyReq) {
     xmlSecGCryptKWAesCtxPtr ctx;
+    int ret;
 
     xmlSecAssert2(xmlSecGCryptKWAesCheckId(transform), -1);
-    xmlSecAssert2((transform->operation == xmlSecTransformOperationEncrypt) || (transform->operation == xmlSecTransformOperationDecrypt), -1);
     xmlSecAssert2(xmlSecTransformCheckSize(transform, xmlSecGCryptKWAesSize), -1);
-    xmlSecAssert2(keyReq != NULL, -1);
 
     ctx = xmlSecGCryptKWAesGetCtx(transform);
     xmlSecAssert2(ctx != NULL, -1);
 
-    keyReq->keyId    = xmlSecGCryptKeyDataAesId;
-    keyReq->keyType  = xmlSecKeyDataTypeSymmetric;
-    if(transform->operation == xmlSecTransformOperationEncrypt) {
-        keyReq->keyUsage = xmlSecKeyUsageEncrypt;
-    } else {
-        keyReq->keyUsage = xmlSecKeyUsageDecrypt;
+    ret = xmlSecTransformKWAesSetKeyReq(transform, &(ctx->parentCtx),keyReq);
+    if(ret < 0) {
+        xmlSecInternalError("xmlSecTransformKWAesSetKeyReq", xmlSecTransformGetName(transform));
+        return(-1);
     }
-    keyReq->keyBitsSize = 8 * ctx->keyExpectedSize;
-
     return(0);
 }
 
 static int
 xmlSecGCryptKWAesSetKey(xmlSecTransformPtr transform, xmlSecKeyPtr key) {
     xmlSecGCryptKWAesCtxPtr ctx;
-    xmlSecBufferPtr buffer;
-    xmlSecSize keySize;
     int ret;
 
     xmlSecAssert2(xmlSecGCryptKWAesCheckId(transform), -1);
-    xmlSecAssert2((transform->operation == xmlSecTransformOperationEncrypt) || (transform->operation == xmlSecTransformOperationDecrypt), -1);
     xmlSecAssert2(xmlSecTransformCheckSize(transform, xmlSecGCryptKWAesSize), -1);
-    xmlSecAssert2(key != NULL, -1);
-    xmlSecAssert2(xmlSecKeyDataCheckId(xmlSecKeyGetValue(key), xmlSecGCryptKeyDataAesId), -1);
 
     ctx = xmlSecGCryptKWAesGetCtx(transform);
     xmlSecAssert2(ctx != NULL, -1);
-
-    buffer = xmlSecKeyDataBinaryValueGetBuffer(xmlSecKeyGetValue(key));
-    xmlSecAssert2(buffer != NULL, -1);
-
-    keySize = xmlSecBufferGetSize(buffer);
-    if(keySize < ctx->keyExpectedSize) {
-        xmlSecInvalidKeyDataSizeError(keySize, ctx->keyExpectedSize,
-                xmlSecTransformGetName(transform));
-        return(-1);
-    }
-
-    ret = xmlSecBufferSetData(&(ctx->keyBuffer),
-                            xmlSecBufferGetData(buffer),
-                            ctx->keyExpectedSize);
+    ret = xmlSecTransformKWAesSetKey(transform, &(ctx->parentCtx), key);
     if(ret < 0) {
-        xmlSecInternalError2("xmlSecBufferSetData",
-                             xmlSecTransformGetName(transform),
-                             "size=%d", ctx->keyExpectedSize);
+        xmlSecInternalError("xmlSecTransformKWAesSetKey", xmlSecTransformGetName(transform));
         return(-1);
     }
-
     return(0);
 }
 
 static int
-xmlSecGCryptKWAesExecute(xmlSecTransformPtr transform, int last, xmlSecTransformCtxPtr transformCtx) {
+xmlSecGCryptKWAesExecute(xmlSecTransformPtr transform, int last,
+                         xmlSecTransformCtxPtr transformCtx ATTRIBUTE_UNUSED) {
     xmlSecGCryptKWAesCtxPtr ctx;
-    xmlSecBufferPtr in, out;
-    xmlSecSize inSize, outSize, keySize;
     int ret;
 
     xmlSecAssert2(xmlSecGCryptKWAesCheckId(transform), -1);
-    xmlSecAssert2((transform->operation == xmlSecTransformOperationEncrypt) || (transform->operation == xmlSecTransformOperationDecrypt), -1);
     xmlSecAssert2(xmlSecTransformCheckSize(transform, xmlSecGCryptKWAesSize), -1);
-    xmlSecAssert2(transformCtx != NULL, -1);
+    UNREFERENCED_PARAMETER(transformCtx);
 
     ctx = xmlSecGCryptKWAesGetCtx(transform);
     xmlSecAssert2(ctx != NULL, -1);
-
-    keySize = xmlSecBufferGetSize(&(ctx->keyBuffer));
-    xmlSecAssert2(keySize == ctx->keyExpectedSize, -1);
-
-    in = &(transform->inBuf);
-    out = &(transform->outBuf);
-    inSize = xmlSecBufferGetSize(in);
-    outSize = xmlSecBufferGetSize(out);
-    xmlSecAssert2(outSize == 0, -1);
-
-    if(transform->status == xmlSecTransformStatusNone) {
-        transform->status = xmlSecTransformStatusWorking;
-    }
-
-    if((transform->status == xmlSecTransformStatusWorking) && (last == 0)) {
-        /* just do nothing */
-    } else  if((transform->status == xmlSecTransformStatusWorking) && (last != 0)) {
-        if((inSize % 8) != 0) {
-            xmlSecInvalidSizeNotMultipleOfError("Input data", inSize, 8,
-                                                xmlSecTransformGetName(transform));
-            return(-1);
-        }
-
-        if(transform->operation == xmlSecTransformOperationEncrypt) {
-            /* the encoded key might be 8 bytes longer plus 8 bytes just in case */
-            outSize = inSize + XMLSEC_KW_AES_MAGIC_BLOCK_SIZE +
-                               XMLSEC_KW_AES_BLOCK_SIZE;
-        } else {
-            outSize = inSize + XMLSEC_KW_AES_BLOCK_SIZE;
-        }
-
-        ret = xmlSecBufferSetMaxSize(out, outSize);
-        if(ret < 0) {
-            xmlSecInternalError2("xmlSecBufferSetMaxSize",
-                                 xmlSecTransformGetName(transform),
-                                 "outSize=%d", outSize);
-            return(-1);
-        }
-
-        if(transform->operation == xmlSecTransformOperationEncrypt) {
-            ret = xmlSecKWAesEncode(&xmlSecGCryptKWAesKlass, ctx,
-                                    xmlSecBufferGetData(in), inSize,
-                                    xmlSecBufferGetData(out), outSize);
-            if(ret < 0) {
-                xmlSecInternalError("xmlSecKWAesEncode",
-                                    xmlSecTransformGetName(transform));
-                return(-1);
-            }
-            outSize = ret;
-        } else {
-            ret = xmlSecKWAesDecode(&xmlSecGCryptKWAesKlass, ctx,
-                                    xmlSecBufferGetData(in), inSize,
-                                    xmlSecBufferGetData(out), outSize);
-            if(ret < 0) {
-                xmlSecInternalError("xmlSecKWAesEncode",
-                                    xmlSecTransformGetName(transform));
-                return(-1);
-            }
-            outSize = ret;
-        }
-
-        ret = xmlSecBufferSetSize(out, outSize);
-        if(ret < 0) {
-            xmlSecInternalError2("xmlSecBufferSetSize",
-                                 xmlSecTransformGetName(transform),
-                                 "outSize=%d", outSize);
-            return(-1);
-        }
-
-        ret = xmlSecBufferRemoveHead(in, inSize);
-        if(ret < 0) {
-            xmlSecInternalError2("xmlSecBufferRemoveHead",
-                                 xmlSecTransformGetName(transform),
-                                 "inSize%d", inSize);
-            return(-1);
-        }
-
-        transform->status = xmlSecTransformStatusFinished;
-    } else if(transform->status == xmlSecTransformStatusFinished) {
-        /* the only way we can get here is if there is no input */
-        xmlSecAssert2(xmlSecBufferGetSize(&(transform->inBuf)) == 0, -1);
-    } else {
-        xmlSecInvalidTransfromStatusError(transform);
+    ret = xmlSecTransformKWAesExecute(transform, &(ctx->parentCtx), last);
+    if(ret < 0) {
+        xmlSecInternalError("xmlSecTransformKWAesExecute", xmlSecTransformGetName(transform));
         return(-1);
     }
+
     return(0);
 }
 
@@ -433,28 +353,40 @@ xmlSecGCryptTransformKWAes256GetKlass(void) {
 static unsigned char g_zero_iv[XMLSEC_KW_AES_BLOCK_SIZE] =
     { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
 static int
-xmlSecGCryptKWAesBlockEncrypt(const xmlSecByte * in, xmlSecSize inSize,
+xmlSecGCryptKWAesBlockEncrypt(xmlSecTransformPtr transform, const xmlSecByte * in, xmlSecSize inSize,
                                xmlSecByte * out, xmlSecSize outSize,
-                               void * context) {
-    xmlSecGCryptKWAesCtxPtr ctx = (xmlSecGCryptKWAesCtxPtr)context;
+                               xmlSecSize * outWritten) {
+    xmlSecGCryptKWAesCtxPtr ctx;
+    xmlSecByte* keyData;
+    xmlSecSize keySize;
     gcry_cipher_hd_t cipherCtx;
     gcry_error_t err;
 
-    xmlSecAssert2(ctx != NULL, -1);
+    xmlSecAssert2(xmlSecGCryptKWAesCheckId(transform), -1);
+    xmlSecAssert2(xmlSecTransformCheckSize(transform, xmlSecGCryptKWAesSize), -1);
     xmlSecAssert2(in != NULL, -1);
-    xmlSecAssert2(inSize >= ctx->blockSize, -1);
     xmlSecAssert2(out != NULL, -1);
+    xmlSecAssert2(outWritten != NULL, -1);
+
+    ctx = xmlSecGCryptKWAesGetCtx(transform);
+    xmlSecAssert2(ctx != NULL, -1);
+    xmlSecAssert2(ctx->blockSize > 0, -1);
+    xmlSecAssert2(inSize >= ctx->blockSize, -1);
     xmlSecAssert2(outSize >= ctx->blockSize, -1);
 
-    err = gcry_cipher_open(&cipherCtx, ctx->cipher, ctx->mode, ctx->flags); 
+    keyData = xmlSecBufferGetData(&(ctx->parentCtx.keyBuffer));
+    keySize = xmlSecBufferGetSize(&(ctx->parentCtx.keyBuffer));
+    xmlSecAssert2(keyData != NULL, -1);
+    xmlSecAssert2(keySize > 0, -1);
+    xmlSecAssert2(ctx->parentCtx.keyExpectedSize == keySize, -1);
+
+    err = gcry_cipher_open(&cipherCtx, ctx->cipher, ctx->mode, ctx->flags);
     if(err != GPG_ERR_NO_ERROR) {
         xmlSecGCryptError("gcry_cipher_open", err, NULL);
         return(-1);
     }
 
-    err = gcry_cipher_setkey(cipherCtx,
-                             xmlSecBufferGetData(&ctx->keyBuffer),
-                             xmlSecBufferGetSize(&ctx->keyBuffer));
+    err = gcry_cipher_setkey(cipherCtx, keyData, keySize);
     if(err != GPG_ERR_NO_ERROR) {
         xmlSecGCryptError("gcry_cipher_setkey", err, NULL);
         gcry_cipher_close(cipherCtx);
@@ -477,32 +409,46 @@ xmlSecGCryptKWAesBlockEncrypt(const xmlSecByte * in, xmlSecSize inSize,
     }
     gcry_cipher_close(cipherCtx);
 
-    return(ctx->blockSize);
+    /* success */
+    (*outWritten) = ctx->blockSize;
+    return(0);
 }
 
 static int
-xmlSecGCryptKWAesBlockDecrypt(const xmlSecByte * in, xmlSecSize inSize,
+xmlSecGCryptKWAesBlockDecrypt(xmlSecTransformPtr transform, const xmlSecByte * in, xmlSecSize inSize,
                                xmlSecByte * out, xmlSecSize outSize,
-                               void * context) {
-    xmlSecGCryptKWAesCtxPtr ctx = (xmlSecGCryptKWAesCtxPtr)context;
+                               xmlSecSize * outWritten) {
+    xmlSecGCryptKWAesCtxPtr ctx;
+    xmlSecByte* keyData;
+    xmlSecSize keySize;
     gcry_cipher_hd_t cipherCtx;
     gcry_error_t err;
 
-    xmlSecAssert2(ctx != NULL, -1);
+    xmlSecAssert2(xmlSecGCryptKWAesCheckId(transform), -1);
+    xmlSecAssert2(xmlSecTransformCheckSize(transform, xmlSecGCryptKWAesSize), -1);
     xmlSecAssert2(in != NULL, -1);
-    xmlSecAssert2(inSize >= ctx->blockSize, -1);
     xmlSecAssert2(out != NULL, -1);
+    xmlSecAssert2(outWritten != NULL, -1);
+
+    ctx = xmlSecGCryptKWAesGetCtx(transform);
+    xmlSecAssert2(ctx != NULL, -1);
+    xmlSecAssert2(ctx->blockSize > 0, -1);
+    xmlSecAssert2(inSize >= ctx->blockSize, -1);
     xmlSecAssert2(outSize >= ctx->blockSize, -1);
 
+    keyData = xmlSecBufferGetData(&(ctx->parentCtx.keyBuffer));
+    keySize = xmlSecBufferGetSize(&(ctx->parentCtx.keyBuffer));
+    xmlSecAssert2(keyData != NULL, -1);
+    xmlSecAssert2(keySize > 0, -1);
+    xmlSecAssert2(ctx->parentCtx.keyExpectedSize == keySize, -1);
+
     err = gcry_cipher_open(&cipherCtx, ctx->cipher, ctx->mode, ctx->flags);
     if(err != GPG_ERR_NO_ERROR) {
         xmlSecGCryptError("gcry_cipher_open", err, NULL);
         return(-1);
     }
 
-    err = gcry_cipher_setkey(cipherCtx,
-                             xmlSecBufferGetData(&ctx->keyBuffer),
-                             xmlSecBufferGetSize(&ctx->keyBuffer));
+    err = gcry_cipher_setkey(cipherCtx, keyData, keySize);
     if(err != GPG_ERR_NO_ERROR) {
         xmlSecGCryptError("gcry_cipher_setkey", err, NULL);
         gcry_cipher_close(cipherCtx);
@@ -525,7 +471,9 @@ xmlSecGCryptKWAesBlockDecrypt(const xmlSecByte * in, xmlSecSize inSize,
     }
     gcry_cipher_close(cipherCtx);
 
-    return(ctx->blockSize);
+    /* success */
+    (*outWritten) = ctx->blockSize;
+    return(0);
 }
 
 #endif /* XMLSEC_NO_AES */
