@@ -4,7 +4,7 @@
  *
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
- * Copyright (C) 2002-2016 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
+ * Copyright (C) 2002-2022 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  */
 /**
  * SECTION:list
@@ -24,6 +24,8 @@
 #include <xmlsec/list.h>
 #include <xmlsec/errors.h>
 
+#include "cast_helpers.h"
+
 static int              xmlSecPtrListEnsureSize                 (xmlSecPtrListPtr list,
                                                                  xmlSecSize size);
 
@@ -183,9 +185,8 @@ xmlSecPtrListCopy(xmlSecPtrListPtr dst, xmlSecPtrListPtr src) {
     /* allocate memory */
     ret = xmlSecPtrListEnsureSize(dst, dst->use + src->use);
     if(ret < 0) {
-        xmlSecInternalError2("xmlSecPtrListEnsureSize",
-                             xmlSecPtrListGetName(src),
-                             "size=%d", src->use);
+        xmlSecInternalError2("xmlSecPtrListEnsureSize", xmlSecPtrListGetName(src),
+            "size=" XMLSEC_SIZE_FMT, src->use);
         return(-1);
     }
 
@@ -197,8 +198,7 @@ xmlSecPtrListCopy(xmlSecPtrListPtr dst, xmlSecPtrListPtr src) {
         if((dst->id->duplicateItem != NULL) && (src->data[i] != NULL)) {
             dst->data[dst->use] = dst->id->duplicateItem(src->data[i]);
             if(dst->data[dst->use] == NULL) {
-                xmlSecInternalError("duplicateItem",
-                                    xmlSecPtrListGetName(src));
+                xmlSecInternalError("duplicateItem", xmlSecPtrListGetName(src));
                 return(-1);
             }
         } else {
@@ -292,9 +292,8 @@ xmlSecPtrListAdd(xmlSecPtrListPtr list, xmlSecPtr item) {
 
     ret = xmlSecPtrListEnsureSize(list, list->use + 1);
     if(ret < 0) {
-        xmlSecInternalError2("xmlSecPtrListEnsureSize",
-                             xmlSecPtrListGetName(list),
-                             "size=%d", list->use + 1);
+        xmlSecInternalError2("xmlSecPtrListEnsureSize", xmlSecPtrListGetName(list),
+            "size=" XMLSEC_SIZE_FMT, list->use + 1);
         return(-1);
     }
 
@@ -389,7 +388,7 @@ xmlSecPtrListDebugDump(xmlSecPtrListPtr list, FILE* output) {
     xmlSecAssert(xmlSecPtrListIsValid(list));
     xmlSecAssert(output != NULL);
 
-    fprintf(output, "=== list size: %d\n", list->use);
+    fprintf(output, "=== list size: " XMLSEC_SIZE_FMT "\n", list->use);
     if(list->id->debugDumpItem != NULL) {
         xmlSecSize pos;
 
@@ -414,7 +413,7 @@ xmlSecPtrListDebugXmlDump(xmlSecPtrListPtr list, FILE* output) {
     xmlSecAssert(xmlSecPtrListIsValid(list));
     xmlSecAssert(output != NULL);
 
-    fprintf(output, "<List size=\"%d\">\n", list->use);
+    fprintf(output, "<List size=\"" XMLSEC_SIZE_FMT "\">\n", list->use);
     if(list->id->debugXmlDumpItem != NULL) {
         xmlSecSize pos;
 
