@@ -5,7 +5,7 @@
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  *
- * Copyright (C) 2002-2016 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
+ * Copyright (C) 2002-2022 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  */
 /**
  * SECTION:kw_des
@@ -25,44 +25,49 @@
 #include <wincrypt.h>
 
 #include <xmlsec/xmlsec.h>
-#include <xmlsec/xmltree.h>
 #include <xmlsec/keys.h>
 #include <xmlsec/transforms.h>
 #include <xmlsec/errors.h>
+#include <xmlsec/private.h>
 
 #include <xmlsec/mscrypto/crypto.h>
 
-#include "../kw_aes_des.h"
 #include "private.h"
 
+#include "../kw_aes_des.h"
+#include "../cast_helpers.h"
 
 /*********************************************************************
  *
  * DES KW implementation
  *
  *********************************************************************/
-static int       xmlSecMSCryptoKWDes3GenerateRandom               (void * context,
-                                                                 xmlSecByte * out, 
-                                                                 xmlSecSize outSize);
-static int       xmlSecMSCryptoKWDes3Sha1                         (void * context,
-                                                                 const xmlSecByte * in, 
-                                                                 xmlSecSize inSize, 
-                                                                 xmlSecByte * out, 
-                                                                 xmlSecSize outSize);
-static int      xmlSecMSCryptoKWDes3BlockEncrypt                  (void * context,
-                                                                 const xmlSecByte * iv, 
-                                                                 xmlSecSize ivSize,
-                                                                 const xmlSecByte * in, 
+static int       xmlSecMSCryptoKWDes3GenerateRandom              (xmlSecTransformPtr transform,
+                                                                 xmlSecByte * out,
+                                                                 xmlSecSize outSize,
+                                                                 xmlSecSize * outWritten);
+static int       xmlSecMSCryptoKWDes3Sha1                        (xmlSecTransformPtr transform,
+                                                                 const xmlSecByte * in,
                                                                  xmlSecSize inSize,
-                                                                 xmlSecByte * out, 
-                                                                 xmlSecSize outSize);
-static int      xmlSecMSCryptoKWDes3BlockDecrypt                  (void * context,
-                                                                 const xmlSecByte * iv, 
+                                                                 xmlSecByte * out,
+                                                                 xmlSecSize outSize,
+                                                                 xmlSecSize * outWritten);
+static int      xmlSecMSCryptoKWDes3BlockEncrypt                 (xmlSecTransformPtr transform,
+                                                                 const xmlSecByte * iv,
                                                                  xmlSecSize ivSize,
-                                                                 const xmlSecByte * in, 
+                                                                 const xmlSecByte * in,
                                                                  xmlSecSize inSize,
-                                                                 xmlSecByte * out, 
-                                                                 xmlSecSize outSize);
+                                                                 xmlSecByte * out,
+                                                                 xmlSecSize outSize,
+                                                                 xmlSecSize * outWritten);
+static int      xmlSecMSCryptoKWDes3BlockDecrypt                 (xmlSecTransformPtr transform,
+                                                                 const xmlSecByte * iv,
+                                                                 xmlSecSize ivSize,
+                                                                 const xmlSecByte * in,
+                                                                 xmlSecSize inSize,
+                                                                 xmlSecByte * out,
+                                                                 xmlSecSize outSize,
+                                                                 xmlSecSize * outWritten);
 
 static xmlSecKWDes3Klass xmlSecMSCryptoKWDes3ImplKlass = {
     /* callbacks */
@@ -74,34 +79,35 @@ static xmlSecKWDes3Klass xmlSecMSCryptoKWDes3ImplKlass = {
     /* for the future */
     NULL,                                   /* void*                               reserved0; */
     NULL,                                   /* void*                               reserved1; */
-}; 
+};
 
 /*********************************************************************
  *
- * Triple DES Key Wrap transform
- *
- * key (xmlSecBuffer) is located after xmlSecTransform structure
+ * Triple DES Key Wrap transform context
  *
  ********************************************************************/
 typedef struct _xmlSecMSCryptoKWDes3Ctx              xmlSecMSCryptoKWDes3Ctx,
                                                   *xmlSecMSCryptoKWDes3CtxPtr;
 struct _xmlSecMSCryptoKWDes3Ctx {
+    xmlSecTransformKWDes3Ctx            parentCtx;
+
     ALG_ID                              desAlgorithmIdentifier;
     const xmlSecMSCryptoProviderInfo  * desProviders;
     ALG_ID                              sha1AlgorithmIdentifier;
     const xmlSecMSCryptoProviderInfo  * sha1Providers;
-    xmlSecKeyDataId                     keyId;
-    xmlSecSize                          keySize;
 
     HCRYPTPROV                          desCryptProvider;
     HCRYPTPROV                          sha1CryptProvider;
     HCRYPTKEY                           pubPrivKey;
-    xmlSecBuffer                        keyBuffer;
 };
-#define xmlSecMSCryptoKWDes3Size     \
-    (sizeof(xmlSecTransform) + sizeof(xmlSecMSCryptoKWDes3Ctx))
-#define xmlSecMSCryptoKWDes3GetCtx(transform) \
-    ((xmlSecMSCryptoKWDes3CtxPtr)(((xmlSecByte*)(transform)) + sizeof(xmlSecTransform)))
+
+/*********************************************************************
+ *
+ * Triple DES Key Wrap transform
+ *
+ ********************************************************************/
+XMLSEC_TRANSFORM_DECLARE(MSCryptoKWDes3, xmlSecMSCryptoKWDes3Ctx)
+#define xmlSecMSCryptoKWDes3Size XMLSEC_TRANSFORM_SIZE(MSCryptoKWDes3)
 
 static int      xmlSecMSCryptoKWDes3Initialize                   (xmlSecTransformPtr transform);
 static void     xmlSecMSCryptoKWDes3Finalize                     (xmlSecTransformPtr transform);
@@ -181,33 +187,26 @@ xmlSecMSCryptoKWDes3Initialize(xmlSecTransformPtr transform) {
 
     ctx = xmlSecMSCryptoKWDes3GetCtx(transform);
     xmlSecAssert2(ctx != NULL, -1);
-
     memset(ctx, 0, sizeof(xmlSecMSCryptoKWDes3Ctx));
 
-    if(transform->id == xmlSecMSCryptoTransformKWDes3Id) {
-        ctx->desAlgorithmIdentifier  = CALG_3DES;
-        ctx->desProviders            = xmlSecMSCryptoProviderInfo_Des;
-        ctx->sha1AlgorithmIdentifier = CALG_SHA1;
-        ctx->sha1Providers           = xmlSecMSCryptoProviderInfo_Sha1;
-        ctx->keyId                   = xmlSecMSCryptoKeyDataDesId;
-        ctx->keySize                 = XMLSEC_KW_DES3_KEY_LENGTH;
-    } else {
-        xmlSecInvalidTransfromError(transform)
+    ret = xmlSecTransformKWDes3Initialize(transform, &(ctx->parentCtx), &xmlSecMSCryptoKWDes3ImplKlass,
+        xmlSecMSCryptoKeyDataDesId);
+    if (ret < 0) {
+        xmlSecInternalError("xmlSecTransformKWDes3Initialize", xmlSecTransformGetName(transform));
         return(-1);
     }
 
-    ret = xmlSecBufferInitialize(&(ctx->keyBuffer), 0);
-    if(ret < 0) {
-        xmlSecInternalError("xmlSecBufferInitialize",
-                            xmlSecTransformGetName(transform));
-        return(-1);
-    }
+    ctx->desAlgorithmIdentifier  = CALG_3DES;
+    ctx->desProviders            = xmlSecMSCryptoProviderInfo_Des;
+    ctx->sha1AlgorithmIdentifier = CALG_SHA1;
+    ctx->sha1Providers           = xmlSecMSCryptoProviderInfo_Sha1;
 
     /* find providers */
     ctx->desCryptProvider = xmlSecMSCryptoFindProvider(ctx->desProviders, NULL, CRYPT_VERIFYCONTEXT, TRUE);
     if(ctx->desCryptProvider == 0) {
         xmlSecInternalError("xmlSecMSCryptoFindProvider(des)",
                             xmlSecTransformGetName(transform));
+        xmlSecMSCryptoKWDes3Finalize(transform);
         return(-1);
     }
 
@@ -215,6 +214,7 @@ xmlSecMSCryptoKWDes3Initialize(xmlSecTransformPtr transform) {
     if(ctx->sha1CryptProvider == 0) {
         xmlSecInternalError("xmlSecMSCryptoFindProvider(sha1)",
                             xmlSecTransformGetName(transform));
+        xmlSecMSCryptoKWDes3Finalize(transform);
         return(-1);
     }
 
@@ -222,6 +222,7 @@ xmlSecMSCryptoKWDes3Initialize(xmlSecTransformPtr transform) {
     if (!xmlSecMSCryptoCreatePrivateExponentOneKey(ctx->desCryptProvider, &(ctx->pubPrivKey))) {
         xmlSecMSCryptoError("xmlSecMSCryptoCreatePrivateExponentOneKey",
                             xmlSecTransformGetName(transform));
+        xmlSecMSCryptoKWDes3Finalize(transform);
         return(-1);
     }
 
@@ -247,174 +248,65 @@ xmlSecMSCryptoKWDes3Finalize(xmlSecTransformPtr transform) {
     if (ctx->sha1CryptProvider) {
         CryptReleaseContext(ctx->sha1CryptProvider, 0);
     }
-    
-    xmlSecBufferFinalize(&ctx->keyBuffer);
 
+    xmlSecTransformKWDes3Finalize(transform, &(ctx->parentCtx));
     memset(ctx, 0, sizeof(xmlSecMSCryptoKWDes3Ctx));
 }
 
 static int
 xmlSecMSCryptoKWDes3SetKeyReq(xmlSecTransformPtr transform,  xmlSecKeyReqPtr keyReq) {
     xmlSecMSCryptoKWDes3CtxPtr ctx;
+    int ret;
 
     xmlSecAssert2(xmlSecTransformCheckId(transform, xmlSecMSCryptoTransformKWDes3Id), -1);
-    xmlSecAssert2((transform->operation == xmlSecTransformOperationEncrypt) || (transform->operation == xmlSecTransformOperationDecrypt), -1);
     xmlSecAssert2(xmlSecTransformCheckSize(transform, xmlSecMSCryptoKWDes3Size), -1);
-    xmlSecAssert2(keyReq != NULL, -1);
 
     ctx = xmlSecMSCryptoKWDes3GetCtx(transform);
     xmlSecAssert2(ctx != NULL, -1);
 
-    keyReq->keyId       = xmlSecMSCryptoKeyDataDesId;
-    keyReq->keyType     = xmlSecKeyDataTypeSymmetric;
-    if(transform->operation == xmlSecTransformOperationEncrypt) {
-        keyReq->keyUsage= xmlSecKeyUsageEncrypt;
-    } else {
-        keyReq->keyUsage= xmlSecKeyUsageDecrypt;
+    ret = xmlSecTransformKWDes3SetKeyReq(transform, &(ctx->parentCtx), keyReq);
+    if (ret < 0) {
+        xmlSecInternalError("xmlSecTransformKWDes3SetKeyReq", xmlSecTransformGetName(transform));
+        return(-1);
     }
-    keyReq->keyBitsSize = 8 * XMLSEC_KW_DES3_KEY_LENGTH;
     return(0);
 }
 
 static int
 xmlSecMSCryptoKWDes3SetKey(xmlSecTransformPtr transform, xmlSecKeyPtr key) {
     xmlSecMSCryptoKWDes3CtxPtr ctx;
-    xmlSecBufferPtr buffer;
-    xmlSecSize keySize;
     int ret;
 
     xmlSecAssert2(xmlSecTransformCheckId(transform, xmlSecMSCryptoTransformKWDes3Id), -1);
-    xmlSecAssert2((transform->operation == xmlSecTransformOperationEncrypt) || (transform->operation == xmlSecTransformOperationDecrypt), -1);
     xmlSecAssert2(xmlSecTransformCheckSize(transform, xmlSecMSCryptoKWDes3Size), -1);
-    xmlSecAssert2(key != NULL, -1);
-    xmlSecAssert2(xmlSecKeyDataCheckId(xmlSecKeyGetValue(key), xmlSecMSCryptoKeyDataDesId), -1);
 
     ctx = xmlSecMSCryptoKWDes3GetCtx(transform);
     xmlSecAssert2(ctx != NULL, -1);
 
-    buffer = xmlSecKeyDataBinaryValueGetBuffer(xmlSecKeyGetValue(key));
-    xmlSecAssert2(buffer != NULL, -1);
-
-    keySize = xmlSecBufferGetSize(buffer);
-    if(keySize < XMLSEC_KW_DES3_KEY_LENGTH) {
-        xmlSecInvalidKeyDataSizeError(keySize, XMLSEC_KW_DES3_KEY_LENGTH,
-                xmlSecTransformGetName(transform));
+    ret = xmlSecTransformKWDes3SetKey(transform, &(ctx->parentCtx), key);
+    if (ret < 0) {
+        xmlSecInternalError("xmlSecTransformKWDes3SetKey", xmlSecTransformGetName(transform));
         return(-1);
     }
-
-    ret = xmlSecBufferSetData(&(ctx->keyBuffer), xmlSecBufferGetData(buffer), XMLSEC_KW_DES3_KEY_LENGTH);
-    if(ret < 0) {
-        xmlSecInternalError2("xmlSecBufferSetData",
-                             xmlSecTransformGetName(transform),
-                             "size=%d", XMLSEC_KW_DES3_KEY_LENGTH);
-        return(-1);
-    }
-
     return(0);
 }
 
 static int
-xmlSecMSCryptoKWDes3Execute(xmlSecTransformPtr transform, int last, xmlSecTransformCtxPtr transformCtx) {
+xmlSecMSCryptoKWDes3Execute(xmlSecTransformPtr transform, int last,
+                            xmlSecTransformCtxPtr transformCtx ATTRIBUTE_UNUSED) {
     xmlSecMSCryptoKWDes3CtxPtr ctx;
-    xmlSecBufferPtr in, out;
-    xmlSecSize inSize, outSize, keySize;
     int ret;
 
     xmlSecAssert2(xmlSecTransformCheckId(transform, xmlSecMSCryptoTransformKWDes3Id), -1);
-    xmlSecAssert2((transform->operation == xmlSecTransformOperationEncrypt) || (transform->operation == xmlSecTransformOperationDecrypt), -1);
     xmlSecAssert2(xmlSecTransformCheckSize(transform, xmlSecMSCryptoKWDes3Size), -1);
-    xmlSecAssert2(transformCtx != NULL, -1);
+    UNREFERENCED_PARAMETER(transformCtx);
 
     ctx = xmlSecMSCryptoKWDes3GetCtx(transform);
     xmlSecAssert2(ctx != NULL, -1);
 
-    keySize = xmlSecBufferGetSize(&(ctx->keyBuffer));
-    xmlSecAssert2(keySize == XMLSEC_KW_DES3_KEY_LENGTH, -1);
-
-    in = &(transform->inBuf);
-    out = &(transform->outBuf);
-    inSize = xmlSecBufferGetSize(in);
-    outSize = xmlSecBufferGetSize(out);
-    xmlSecAssert2(outSize == 0, -1);
-
-    if(transform->status == xmlSecTransformStatusNone) {
-        transform->status = xmlSecTransformStatusWorking;
-    }
-
-    if((transform->status == xmlSecTransformStatusWorking) && (last == 0)) {
-        /* just do nothing */
-    } else  if((transform->status == xmlSecTransformStatusWorking) && (last != 0)) {
-        if((inSize % XMLSEC_KW_DES3_BLOCK_LENGTH) != 0) {
-            xmlSecInvalidSizeNotMultipleOfError("Input data",
-                    inSize, XMLSEC_KW_DES3_BLOCK_LENGTH,
-                    xmlSecTransformGetName(transform));
-            return(-1);
-        }
-
-        if(transform->operation == xmlSecTransformOperationEncrypt) {
-            /* the encoded key might be 16 bytes longer plus one block just in case */
-            outSize = inSize + XMLSEC_KW_DES3_IV_LENGTH +
-                               XMLSEC_KW_DES3_BLOCK_LENGTH +
-                               XMLSEC_KW_DES3_BLOCK_LENGTH;
-        } else {
-            /* just in case, add a block */
-            outSize = inSize + XMLSEC_KW_DES3_BLOCK_LENGTH;
-        }
-
-        ret = xmlSecBufferSetMaxSize(out, outSize);
-        if(ret < 0) {
-            xmlSecInternalError2("xmlSecBufferSetMaxSize",
-                                 xmlSecTransformGetName(transform),
-                                 "size=%d", outSize);
-            return(-1);
-        }
-
-        if(transform->operation == xmlSecTransformOperationEncrypt) {
-            ret = xmlSecKWDes3Encode(&xmlSecMSCryptoKWDes3ImplKlass, ctx,
-                                    xmlSecBufferGetData(in), inSize,
-                                    xmlSecBufferGetData(out), outSize);
-            if(ret < 0) {
-                xmlSecInternalError4("xmlSecKWDes3Encode", xmlSecTransformGetName(transform),
-                                     "key=%d,in=%d,out=%d",
-                                    keySize, inSize, outSize);
-                return(-1);
-            }
-            outSize = ret;
-        } else {
-            ret = xmlSecKWDes3Decode(&xmlSecMSCryptoKWDes3ImplKlass, ctx,
-                                    xmlSecBufferGetData(in), inSize,
-                                    xmlSecBufferGetData(out), outSize);
-            if(ret < 0) {
-                xmlSecInternalError4("xmlSecKWDes3Decode", xmlSecTransformGetName(transform),
-                                     "key=%d,in=%d,out=%d",
-                                     keySize, inSize, outSize);
-                return(-1);
-            }
-            outSize = ret;
-        }
-
-        ret = xmlSecBufferSetSize(out, outSize);
-        if(ret < 0) {
-            xmlSecInternalError2("xmlSecBufferSetSize",
-                                 xmlSecTransformGetName(transform),
-                                 "size=%d", outSize);
-            return(-1);
-        }
-
-        ret = xmlSecBufferRemoveHead(in, inSize);
-        if(ret < 0) {
-            xmlSecInternalError2("xmlSecBufferRemoveHead",
-                                 xmlSecTransformGetName(transform),
-                                 "size=%d", inSize);
-            return(-1);
-        }
-
-        transform->status = xmlSecTransformStatusFinished;
-    } else if(transform->status == xmlSecTransformStatusFinished) {
-        /* the only way we can get here is if there is no input */
-        xmlSecAssert2(xmlSecBufferGetSize(&(transform->inBuf)) == 0, -1);
-    } else {
-        xmlSecInvalidTransfromStatusError(transform);
+    ret = xmlSecTransformKWDes3Execute(transform, &(ctx->parentCtx), last);
+    if (ret < 0) {
+        xmlSecInternalError("xmlSecTransformKWDes3Execute", xmlSecTransformGetName(transform));
         return(-1);
     }
     return(0);
@@ -426,21 +318,28 @@ xmlSecMSCryptoKWDes3Execute(xmlSecTransformPtr transform, int last, xmlSecTransf
  *
  *********************************************************************/
 static int
-xmlSecMSCryptoKWDes3Sha1(void * context,
-                       const xmlSecByte * in, xmlSecSize inSize, 
-                       xmlSecByte * out, xmlSecSize outSize) {
-    xmlSecMSCryptoKWDes3CtxPtr ctx = (xmlSecMSCryptoKWDes3CtxPtr)context;
+xmlSecMSCryptoKWDes3Sha1(xmlSecTransformPtr transform,
+                       const xmlSecByte * in, xmlSecSize inSize,
+                       xmlSecByte * out, xmlSecSize outSize,
+                       xmlSecSize* outWritten) {
+    xmlSecMSCryptoKWDes3CtxPtr ctx;
     HCRYPTHASH mscHash = 0;
-    DWORD retLen;
+    DWORD dwInSize, dwOutSize;
     int ret;
+    int res = -1;
 
-    xmlSecAssert2(ctx != NULL, -1);
-    xmlSecAssert2(ctx->sha1CryptProvider != 0, -1);
-    xmlSecAssert2(ctx->sha1AlgorithmIdentifier != 0, -1);
+    xmlSecAssert2(xmlSecTransformCheckId(transform, xmlSecMSCryptoTransformKWDes3Id), -1);
+    xmlSecAssert2(xmlSecTransformCheckSize(transform, xmlSecMSCryptoKWDes3Size), -1);
     xmlSecAssert2(in != NULL, -1);
     xmlSecAssert2(inSize > 0, -1);
     xmlSecAssert2(out != NULL, -1);
     xmlSecAssert2(outSize > 0, -1);
+    xmlSecAssert2(outWritten != NULL, -1);
+
+    ctx = xmlSecMSCryptoKWDes3GetCtx(transform);
+    xmlSecAssert2(ctx != NULL, -1);
+    xmlSecAssert2(ctx->sha1CryptProvider != 0, -1);
+    xmlSecAssert2(ctx->sha1AlgorithmIdentifier != 0, -1);
 
     /* create */
     ret = CryptCreateHash(ctx->sha1CryptProvider,
@@ -450,115 +349,128 @@ xmlSecMSCryptoKWDes3Sha1(void * context,
         &mscHash);
     if((ret == 0) || (mscHash == 0)) {
         xmlSecMSCryptoError("CryptCreateHash", NULL);
-        return(-1);
+        goto done;
     }
 
     /* hash */
-    ret = CryptHashData(mscHash,
-        in, 
-        inSize,
-        0);
+    XMLSEC_SAFE_CAST_SIZE_TO_ULONG(inSize, dwInSize, goto done, NULL);
+    ret = CryptHashData(mscHash, in,  dwInSize, 0);
     if(ret == 0) {
-        xmlSecMSCryptoError2("CryptHashData", NULL,
-                             "size=%d", inSize);
-        CryptDestroyHash(mscHash);
-        return(-1);
+        xmlSecMSCryptoError2("CryptHashData", NULL, "size=" XMLSEC_SIZE_FMT, inSize);
+        goto done;
     }
 
     /* get results */
-    retLen = outSize;
-    ret = CryptGetHashParam(mscHash,
-        HP_HASHVAL,
-        out,
-        &retLen,
-        0);
+    XMLSEC_SAFE_CAST_SIZE_TO_ULONG(outSize, dwOutSize, goto done, NULL);
+    ret = CryptGetHashParam(mscHash, HP_HASHVAL, out, &dwOutSize, 0);
     if (ret == 0) {
-        xmlSecMSCryptoError2("CryptGetHashParam(HP_HASHVAL)", NULL,
-                             "size=%d", outSize);
-        CryptDestroyHash(mscHash);
-        return(-1);
+        xmlSecMSCryptoError2("CryptGetHashParam(HP_HASHVAL)", NULL, "size=" XMLSEC_SIZE_FMT, outSize);
+        goto done;
     }
 
-    /* done */
-    CryptDestroyHash(mscHash);
-    return(retLen);
+    /* success */
+    XMLSEC_SAFE_CAST_ULONG_TO_SIZE(dwOutSize, (*outWritten), goto done, NULL);
+    res = 0;
+
+done:
+    /* cleanup */
+    if (mscHash != 0) {
+        CryptDestroyHash(mscHash);
+    }
+    return(res);
 }
 
 static int
-xmlSecMSCryptoKWDes3GenerateRandom(void * context,
-                                   xmlSecByte * out, xmlSecSize outSize) 
+xmlSecMSCryptoKWDes3GenerateRandom(xmlSecTransformPtr transform, xmlSecByte * out, xmlSecSize outSize,
+    xmlSecSize* outWritten)
 {
-    xmlSecMSCryptoKWDes3CtxPtr ctx = (xmlSecMSCryptoKWDes3CtxPtr)context;
-    int ret;
+    xmlSecMSCryptoKWDes3CtxPtr ctx;
+    DWORD dwOutSize;
 
-    xmlSecAssert2(ctx != NULL, -1);
-    xmlSecAssert2(ctx->desCryptProvider != 0, -1);
+    xmlSecAssert2(xmlSecTransformCheckId(transform, xmlSecMSCryptoTransformKWDes3Id), -1);
+    xmlSecAssert2(xmlSecTransformCheckSize(transform, xmlSecMSCryptoKWDes3Size), -1);
     xmlSecAssert2(out != NULL, -1);
     xmlSecAssert2(outSize > 0, -1);
+    xmlSecAssert2(outWritten != NULL, -1);
 
-    if(!CryptGenRandom(ctx->desCryptProvider, outSize, out)) {
-        xmlSecMSCryptoError2("CryptGenRandom", NULL,
-                             "len=%d", outSize);
+    ctx = xmlSecMSCryptoKWDes3GetCtx(transform);
+    xmlSecAssert2(ctx != NULL, -1);
+    xmlSecAssert2(ctx->desCryptProvider != 0, -1);
+
+    XMLSEC_SAFE_CAST_SIZE_TO_ULONG(outSize, dwOutSize, return(-1), NULL);
+    if(!CryptGenRandom(ctx->desCryptProvider, dwOutSize, out)) {
+        xmlSecMSCryptoError2("CryptGenRandom", NULL, "len=" XMLSEC_SIZE_FMT, outSize);
         return(-1);
     }
+    (*outWritten) = outSize;
+    return(0);
 
-    return((int)outSize);
 }
 
 static int
-xmlSecMSCryptoKWDes3BlockEncrypt(void * context,
-                               const xmlSecByte * iv, xmlSecSize ivSize,
-                               const xmlSecByte * in, xmlSecSize inSize,
-                               xmlSecByte * out, xmlSecSize outSize) {
-    xmlSecMSCryptoKWDes3CtxPtr ctx = (xmlSecMSCryptoKWDes3CtxPtr)context;
-    DWORD dwBlockLen, dwBlockLenLen, dwCLen;
+xmlSecMSCryptoKWDes3BlockEncrypt(xmlSecTransformPtr transform,
+                                const xmlSecByte * iv, xmlSecSize ivSize,
+                                const xmlSecByte * in, xmlSecSize inSize,
+                                xmlSecByte * out, xmlSecSize outSize,
+                                xmlSecSize* outWritten) {
+    xmlSecMSCryptoKWDes3CtxPtr ctx;
+    xmlSecByte* keyBuf;
+    xmlSecSize keyBufSize, blockSizeInBits;
+    DWORD dwKeyBufSize, dwBlockLen, dwBlockLenLen, dwCLen, dwOutSize;
     HCRYPTKEY cryptKey = 0;
-    int ret;
+    int res = -1;
 
-    xmlSecAssert2(ctx != NULL, -1);
-    xmlSecAssert2(xmlSecBufferGetData(&(ctx->keyBuffer)) != NULL, -1);
-    xmlSecAssert2(xmlSecBufferGetSize(&(ctx->keyBuffer)) >= XMLSEC_KW_DES3_KEY_LENGTH, -1);
+    xmlSecAssert2(xmlSecTransformCheckId(transform, xmlSecMSCryptoTransformKWDes3Id), -1);
+    xmlSecAssert2(xmlSecTransformCheckSize(transform, xmlSecMSCryptoKWDes3Size), -1);
     xmlSecAssert2(iv != NULL, -1);
     xmlSecAssert2(ivSize >= XMLSEC_KW_DES3_IV_LENGTH, -1);
     xmlSecAssert2(in != NULL, -1);
     xmlSecAssert2(inSize > 0, -1);
     xmlSecAssert2(out != NULL, -1);
     xmlSecAssert2(outSize >= inSize, -1);
+    xmlSecAssert2(outWritten != NULL, -1);
 
-    /* Import this key and get an HCRYPTKEY handle, we do it again and again 
+    ctx = xmlSecMSCryptoKWDes3GetCtx(transform);
+    xmlSecAssert2(ctx != NULL, -1);
+
+    keyBuf = xmlSecBufferGetData(&(ctx->parentCtx.keyBuffer));
+    keyBufSize = xmlSecBufferGetSize(&(ctx->parentCtx.keyBuffer));
+    xmlSecAssert2(keyBuf  != NULL, -1);
+    xmlSecAssert2(keyBufSize >= XMLSEC_KW_DES3_KEY_LENGTH, -1);
+    XMLSEC_SAFE_CAST_SIZE_TO_ULONG(keyBufSize, dwKeyBufSize, goto done, NULL);
+
+    /* Import this key and get an HCRYPTKEY handle, we do it again and again
        to ensure we don't go into CBC mode */
     if (!xmlSecMSCryptoImportPlainSessionBlob(ctx->desCryptProvider,
         ctx->pubPrivKey,
         ctx->desAlgorithmIdentifier,
-        xmlSecBufferGetData(&ctx->keyBuffer),
-        xmlSecBufferGetSize(&ctx->keyBuffer),
+        keyBuf,
+        dwKeyBufSize,
         TRUE,
         &cryptKey))  {
 
         xmlSecInternalError("xmlSecMSCryptoImportPlainSessionBlob", NULL);
-        return(-1);
+        goto done;
     }
     xmlSecAssert2(cryptKey != 0, -1);
 
     /* iv len == block len */
-    dwBlockLenLen = sizeof(DWORD);
+    dwBlockLenLen = sizeof(dwBlockLen);
     if (!CryptGetKeyParam(cryptKey, KP_BLOCKLEN, (BYTE *)&dwBlockLen, &dwBlockLenLen, 0)) {
         xmlSecMSCryptoError("CryptGetKeyParam", NULL);
-        CryptDestroyKey(cryptKey);
-        return(-1);
+        goto done;
     }
+    XMLSEC_SAFE_CAST_ULONG_TO_SIZE(dwBlockLen, blockSizeInBits, goto done, NULL);
 
     /* set IV */
-    if(ivSize < dwBlockLen / 8) {
-        xmlSecInvalidSizeLessThanError("ivSize", ivSize, dwBlockLen / 8, NULL);
-        CryptDestroyKey(cryptKey);
-        return(-1);
+    if(ivSize < blockSizeInBits / 8) {
+        xmlSecInvalidSizeLessThanError("ivSize", ivSize, blockSizeInBits / 8, NULL);
+        goto done;
     }
 
     if(!CryptSetKeyParam(cryptKey, KP_IV, iv, 0)) {
         xmlSecMSCryptoError("CryptSetKeyParam", NULL);
-        CryptDestroyKey(cryptKey);
-        return(-1);
+        goto done;
     }
 
     /* Set process last block to false, since we handle padding ourselves, and MSCrypto padding
@@ -566,71 +478,88 @@ xmlSecMSCryptoKWDes3BlockEncrypt(void * context,
     if(out != in) {
         memcpy(out, in, inSize);
     }
-    dwCLen = inSize;
-    if(!CryptEncrypt(cryptKey, 0, FALSE, 0, out, &dwCLen, outSize)) {
+
+    XMLSEC_SAFE_CAST_SIZE_TO_ULONG(inSize, dwCLen, goto done, NULL);
+    XMLSEC_SAFE_CAST_SIZE_TO_ULONG(outSize, dwOutSize, goto done, NULL);
+    if(!CryptEncrypt(cryptKey, 0, FALSE, 0, out, &dwCLen, dwOutSize)) {
         xmlSecMSCryptoError("CryptEncrypt", NULL);
-        CryptDestroyKey(cryptKey);
-        return(-1);
+        goto done;
     }
 
+    /* success */
+    XMLSEC_SAFE_CAST_ULONG_TO_SIZE(dwCLen, (*outWritten), goto done, NULL);
+    res = 0;
+
     /* cleanup */
-    CryptDestroyKey(cryptKey);
-    return(dwCLen);
+done:
+    if (cryptKey != 0) {
+        CryptDestroyKey(cryptKey);
+    }
+    return(res);
 }
 
 static int
-xmlSecMSCryptoKWDes3BlockDecrypt(void * context,
+xmlSecMSCryptoKWDes3BlockDecrypt(xmlSecTransformPtr transform,
                                const xmlSecByte * iv, xmlSecSize ivSize,
                                const xmlSecByte * in, xmlSecSize inSize,
-                               xmlSecByte * out, xmlSecSize outSize) {
-    xmlSecMSCryptoKWDes3CtxPtr ctx = (xmlSecMSCryptoKWDes3CtxPtr)context;
-    DWORD dwBlockLen, dwBlockLenLen, dwCLen;
+                               xmlSecByte * out, xmlSecSize outSize,
+                               xmlSecSize* outWritten) {
+    xmlSecMSCryptoKWDes3CtxPtr ctx;
+    xmlSecByte* keyBuf;
+    xmlSecSize keyBufSize, blockSizeInBits;
+    DWORD dwKeyBufSize, dwBlockLen, dwBlockLenLen, dwCLen;
     HCRYPTKEY cryptKey = 0;
-    int ret;
+    int res = -1;
 
-    xmlSecAssert2(ctx != NULL, -1);
-    xmlSecAssert2(xmlSecBufferGetData(&(ctx->keyBuffer)) != NULL, -1);
-    xmlSecAssert2(xmlSecBufferGetSize(&(ctx->keyBuffer)) >= XMLSEC_KW_DES3_KEY_LENGTH, -1);
+    xmlSecAssert2(xmlSecTransformCheckId(transform, xmlSecMSCryptoTransformKWDes3Id), -1);
+    xmlSecAssert2(xmlSecTransformCheckSize(transform, xmlSecMSCryptoKWDes3Size), -1);
     xmlSecAssert2(iv != NULL, -1);
     xmlSecAssert2(ivSize >= XMLSEC_KW_DES3_IV_LENGTH, -1);
     xmlSecAssert2(in != NULL, -1);
     xmlSecAssert2(inSize > 0, -1);
     xmlSecAssert2(out != NULL, -1);
     xmlSecAssert2(outSize >= inSize, -1);
+    xmlSecAssert2(outWritten != NULL, -1);
 
-    /* Import this key and get an HCRYPTKEY handle, we do it again and again 
+    ctx = xmlSecMSCryptoKWDes3GetCtx(transform);
+    xmlSecAssert2(ctx != NULL, -1);
+
+    keyBuf = xmlSecBufferGetData(&(ctx->parentCtx.keyBuffer));
+    keyBufSize = xmlSecBufferGetSize(&(ctx->parentCtx.keyBuffer));
+    xmlSecAssert2(keyBuf != NULL, -1);
+    xmlSecAssert2(keyBufSize >= XMLSEC_KW_DES3_KEY_LENGTH, -1);
+    XMLSEC_SAFE_CAST_SIZE_TO_ULONG(keyBufSize, dwKeyBufSize, goto done, NULL);
+
+    /* Import this key and get an HCRYPTKEY handle, we do it again and again
        to ensure we don't go into CBC mode */
     if (!xmlSecMSCryptoImportPlainSessionBlob(ctx->desCryptProvider,
         ctx->pubPrivKey,
         ctx->desAlgorithmIdentifier,
-        xmlSecBufferGetData(&ctx->keyBuffer),
-        xmlSecBufferGetSize(&ctx->keyBuffer),
+        keyBuf,
+        dwKeyBufSize,
         TRUE,
         &cryptKey))  {
 
-        xmlSecInternalError("xmlSecMSCryptoImportPlainSessionBlob", NULL);
-        return(-1);
+        goto done;
     }
     xmlSecAssert2(cryptKey != 0, -1);
 
     /* iv len == block len */
-    dwBlockLenLen = sizeof(DWORD);
+    dwBlockLenLen = sizeof(dwBlockLen);
     if (!CryptGetKeyParam(cryptKey, KP_BLOCKLEN, (BYTE *)&dwBlockLen, &dwBlockLenLen, 0)) {
         xmlSecMSCryptoError("CryptGetKeyParam", NULL);
-        CryptDestroyKey(cryptKey);
-        return(-1);
+        goto done;
     }
+    XMLSEC_SAFE_CAST_ULONG_TO_SIZE(dwBlockLen, blockSizeInBits, goto done, NULL);
 
     /* set IV */
-    if(ivSize < dwBlockLen / 8) {
-        xmlSecInvalidSizeLessThanError("ivSize", ivSize, dwBlockLen / 8, NULL);
-        CryptDestroyKey(cryptKey);
-        return(-1);
+    if(ivSize < blockSizeInBits / 8) {
+        xmlSecInvalidSizeLessThanError("ivSize", ivSize, blockSizeInBits / 8, NULL);
+        goto done;
     }
     if(!CryptSetKeyParam(cryptKey, KP_IV, iv, 0)) {
         xmlSecMSCryptoError("CryptSetKeyParam", NULL);
-        CryptDestroyKey(cryptKey);
-        return(-1);
+        goto done;
     }
 
     /* Set process last block to false, since we handle padding ourselves, and MSCrypto padding
@@ -638,16 +567,23 @@ xmlSecMSCryptoKWDes3BlockDecrypt(void * context,
     if(out != in) {
         memcpy(out, in, inSize);
     }
-    dwCLen = inSize;
+
+    XMLSEC_SAFE_CAST_SIZE_TO_ULONG(inSize, dwCLen, goto done, NULL);
     if(!CryptDecrypt(cryptKey, 0, FALSE, 0, out, &dwCLen)) {
         xmlSecMSCryptoError("CryptEncrypt", NULL);
-        CryptDestroyKey(cryptKey);
-        return(-1);
+        goto done;
     }
 
+    /* success */
+    XMLSEC_SAFE_CAST_ULONG_TO_SIZE(dwCLen, (*outWritten), goto done, NULL);
+    res = 0;
+
+done:
     /* cleanup */
-    CryptDestroyKey(cryptKey);
-    return(dwCLen);
+    if (cryptKey != 0) {
+        CryptDestroyKey(cryptKey);
+    }
+    return(res);
 }
 
 
