@@ -5,7 +5,7 @@
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  *
- * Copyright (C) 2002-2016 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
+ * Copyright (C) 2002-2022 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  */
 /**
  * SECTION:hmac
@@ -30,51 +30,64 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <ctype.h>
 
 #include <openssl/hmac.h>
 
 #include <xmlsec/xmlsec.h>
-#include <xmlsec/xmltree.h>
 #include <xmlsec/base64.h>
 #include <xmlsec/keys.h>
-#include <xmlsec/transforms.h>
 #include <xmlsec/errors.h>
+#include <xmlsec/private.h>
 
 #include <xmlsec/openssl/crypto.h>
 #include "openssl_compat.h"
 
-/* sizes in bits */
-#define XMLSEC_OPENSSL_MIN_HMAC_SIZE            80
-#define XMLSEC_OPENSSL_MAX_HMAC_SIZE            (EVP_MAX_MD_SIZE * 8)
+#ifdef XMLSEC_OPENSSL_API_300
+#include <openssl/core_names.h>
+#include <openssl/param_build.h>
+#endif /* XMLSEC_OPENSSL_API_300 */
+
+#include "../cast_helpers.h"
+#include "../keysdata_helpers.h"
+#include "../transform_helpers.h"
+
 
 /**************************************************************************
  *
  * Configuration
  *
  *****************************************************************************/
-static int g_xmlsec_openssl_hmac_min_length = XMLSEC_OPENSSL_MIN_HMAC_SIZE;
 
 /**
  * xmlSecOpenSSLHmacGetMinOutputLength:
  *
+ * DEPRECATED (use @xmlSecTransformHmacGetMinOutputBitsSize instead).
  * Gets the value of min HMAC length.
  *
  * Returns: the min HMAC output length
  */
 int xmlSecOpenSSLHmacGetMinOutputLength(void)
 {
-    return g_xmlsec_openssl_hmac_min_length;
+    xmlSecSize val = xmlSecTransformHmacGetMinOutputBitsSize();
+    int res;
+
+    XMLSEC_SAFE_CAST_SIZE_TO_INT(val, res, return(-1), NULL);
+    return res;
 }
 
 /**
  * xmlSecOpenSSLHmacSetMinOutputLength:
  * @min_length: the new min length
  *
+ * DEPRECATED (use @xmlSecTransformHmacSetMinOutputBitsSize instead).
  * Sets the min HMAC output length
  */
 void xmlSecOpenSSLHmacSetMinOutputLength(int min_length)
 {
-    g_xmlsec_openssl_hmac_min_length = min_length;
+    xmlSecSize val;
+    XMLSEC_SAFE_CAST_INT_TO_SIZE(min_length, val, return, NULL);
+    xmlSecTransformHmacSetMinOutputBitsSize(val);
 }
 
 /**************************************************************************
@@ -84,10 +97,16 @@ void xmlSecOpenSSLHmacSetMinOutputLength(int min_length)
  *****************************************************************************/
 typedef struct _xmlSecOpenSSLHmacCtx            xmlSecOpenSSLHmacCtx, *xmlSecOpenSSLHmacCtxPtr;
 struct _xmlSecOpenSSLHmacCtx {
+#ifndef XMLSEC_OPENSSL_API_300
     const EVP_MD*       hmacDgst;
     HMAC_CTX*           hmacCtx;
+#else /* XMLSEC_OPENSSL_API_300 */
+    const char*         evpHmacDgstName;
+    EVP_MAC*            evpHmac;
+    EVP_MAC_CTX*        evpHmacCtx;
+#endif /* XMLSEC_OPENSSL_API_300 */
     int                 ctxInitialized;
-    xmlSecByte          dgst[XMLSEC_OPENSSL_MAX_HMAC_SIZE];
+    xmlSecByte          dgst[EVP_MAX_MD_SIZE];
     xmlSecSize          dgstSize;       /* dgst size in bits */
 };
 
@@ -95,13 +114,9 @@ struct _xmlSecOpenSSLHmacCtx {
  *
  * HMAC transforms
  *
- * xmlSecOpenSSLHmacCtx is located after xmlSecTransform
- *
  *****************************************************************************/
-#define xmlSecOpenSSLHmacGetCtx(transform) \
-    ((xmlSecOpenSSLHmacCtxPtr)(((xmlSecByte*)(transform)) + sizeof(xmlSecTransform)))
-#define xmlSecOpenSSLHmacSize   \
-    (sizeof(xmlSecTransform) + sizeof(xmlSecOpenSSLHmacCtx))
+XMLSEC_TRANSFORM_DECLARE(OpenSSLHmac, xmlSecOpenSSLHmacCtx)
+#define xmlSecOpenSSLHmacSize XMLSEC_TRANSFORM_SIZE(OpenSSLHmac)
 
 static int      xmlSecOpenSSLHmacCheckId                        (xmlSecTransformPtr transform);
 static int      xmlSecOpenSSLHmacInitialize                     (xmlSecTransformPtr transform);
@@ -173,7 +188,14 @@ xmlSecOpenSSLHmacCheckId(xmlSecTransformPtr transform) {
     }
 }
 
-
+/* small helper macro to reduce clutter in the code */
+#ifndef XMLSEC_OPENSSL_API_300
+#define XMLSEC_OPENSSL_HMAC_SET_DIGEST(ctx, digestVal, digestNameVal) \
+    (ctx)->hmacDgst = (digestVal)
+#else /* XMLSEC_OPENSSL_API_300 */
+#define XMLSEC_OPENSSL_HMAC_SET_DIGEST(ctx, digestVal, digestNameVal) \
+    (ctx)->evpHmacDgstName = (digestNameVal)
+#endif /* XMLSEC_OPENSSL_API_300 */
 
 static int
 xmlSecOpenSSLHmacInitialize(xmlSecTransformPtr transform) {
@@ -190,43 +212,43 @@ xmlSecOpenSSLHmacInitialize(xmlSecTransformPtr transform) {
 
 #ifndef XMLSEC_NO_SHA1
     if(xmlSecTransformCheckId(transform, xmlSecOpenSSLTransformHmacSha1Id)) {
-        ctx->hmacDgst = EVP_sha1();
+        XMLSEC_OPENSSL_HMAC_SET_DIGEST(ctx, EVP_sha1(), OSSL_DIGEST_NAME_SHA1);
     } else
 #endif /* XMLSEC_NO_SHA1 */
 
 #ifndef XMLSEC_NO_SHA224
     if(xmlSecTransformCheckId(transform, xmlSecOpenSSLTransformHmacSha224Id)) {
-        ctx->hmacDgst = EVP_sha224();
+        XMLSEC_OPENSSL_HMAC_SET_DIGEST(ctx, EVP_sha224(), OSSL_DIGEST_NAME_SHA2_224);
     } else
 #endif /* XMLSEC_NO_SHA224 */
 
 #ifndef XMLSEC_NO_SHA256
     if(xmlSecTransformCheckId(transform, xmlSecOpenSSLTransformHmacSha256Id)) {
-        ctx->hmacDgst = EVP_sha256();
+        XMLSEC_OPENSSL_HMAC_SET_DIGEST(ctx, EVP_sha256(), OSSL_DIGEST_NAME_SHA2_256);
     } else
 #endif /* XMLSEC_NO_SHA256 */
 
 #ifndef XMLSEC_NO_SHA384
     if(xmlSecTransformCheckId(transform, xmlSecOpenSSLTransformHmacSha384Id)) {
-        ctx->hmacDgst = EVP_sha384();
+        XMLSEC_OPENSSL_HMAC_SET_DIGEST(ctx, EVP_sha384(), OSSL_DIGEST_NAME_SHA2_384);
     } else
 #endif /* XMLSEC_NO_SHA384 */
 
 #ifndef XMLSEC_NO_SHA512
     if(xmlSecTransformCheckId(transform, xmlSecOpenSSLTransformHmacSha512Id)) {
-        ctx->hmacDgst = EVP_sha512();
+        XMLSEC_OPENSSL_HMAC_SET_DIGEST(ctx, EVP_sha512(), OSSL_DIGEST_NAME_SHA2_512);
     } else
 #endif /* XMLSEC_NO_SHA512 */
 
 #ifndef XMLSEC_NO_RIPEMD160
     if(xmlSecTransformCheckId(transform, xmlSecOpenSSLTransformHmacRipemd160Id)) {
-        ctx->hmacDgst = EVP_ripemd160();
+        XMLSEC_OPENSSL_HMAC_SET_DIGEST(ctx, EVP_ripemd160(), OSSL_DIGEST_NAME_RIPEMD160);
     } else
 #endif /* XMLSEC_NO_RIPEMD160 */
 
 #ifndef XMLSEC_NO_MD5
     if(xmlSecTransformCheckId(transform, xmlSecOpenSSLTransformHmacMd5Id)) {
-        ctx->hmacDgst = EVP_md5();
+        XMLSEC_OPENSSL_HMAC_SET_DIGEST(ctx, EVP_md5(), OSSL_DIGEST_NAME_MD5);
     } else
 #endif /* XMLSEC_NO_MD5 */
 
@@ -235,13 +257,28 @@ xmlSecOpenSSLHmacInitialize(xmlSecTransformPtr transform) {
         return(-1);
     }
 
+#ifndef XMLSEC_OPENSSL_API_300
     /* create hmac CTX */
     ctx->hmacCtx = HMAC_CTX_new();
     if(ctx->hmacCtx == NULL) {
-        xmlSecOpenSSLError("HMAC_CTX_new",
-                           xmlSecTransformGetName(transform));
+        xmlSecOpenSSLError("HMAC_CTX_new", xmlSecTransformGetName(transform));
+        xmlSecOpenSSLHmacFinalize(transform);
         return(-1);
     }
+#else /* XMLSEC_OPENSSL_API_300 */
+    ctx->evpHmac = EVP_MAC_fetch(xmlSecOpenSSLGetLibCtx(), OSSL_MAC_NAME_HMAC, NULL);
+    if (ctx->evpHmac == NULL) {
+        xmlSecOpenSSLError("EVP_MAC_fetch", xmlSecTransformGetName(transform));
+        xmlSecOpenSSLHmacFinalize(transform);
+        return(-1);
+    }
+    ctx->evpHmacCtx = EVP_MAC_CTX_new(ctx->evpHmac);
+    if (ctx->evpHmacCtx == NULL) {
+        xmlSecOpenSSLError("EVP_MAC_CTX_new", xmlSecTransformGetName(transform));
+        xmlSecOpenSSLHmacFinalize(transform);
+        return(-1);
+    }
+#endif /* XMLSEC_OPENSSL_API_300 */
 
     /* done */
     return(0);
@@ -257,53 +294,43 @@ xmlSecOpenSSLHmacFinalize(xmlSecTransformPtr transform) {
     ctx = xmlSecOpenSSLHmacGetCtx(transform);
     xmlSecAssert(ctx != NULL);
 
+#ifndef XMLSEC_OPENSSL_API_300
     if(ctx->hmacCtx != NULL) {
         HMAC_CTX_free(ctx->hmacCtx);
     }
+#else /* XMLSEC_OPENSSL_API_300 */
+    if(ctx->evpHmacCtx != NULL) {
+        EVP_MAC_CTX_free(ctx->evpHmacCtx);
+    }
+    if (ctx->evpHmac != NULL) {
+        EVP_MAC_free(ctx->evpHmac);
+    }
+#endif /* XMLSEC_OPENSSL_API_300 */
 
     memset(ctx, 0, sizeof(xmlSecOpenSSLHmacCtx));
 }
 
 static int
-xmlSecOpenSSLHmacNodeRead(xmlSecTransformPtr transform, xmlNodePtr node, xmlSecTransformCtxPtr transformCtx) {
+xmlSecOpenSSLHmacNodeRead(xmlSecTransformPtr transform, xmlNodePtr node,
+                          xmlSecTransformCtxPtr transformCtx ATTRIBUTE_UNUSED) {
     xmlSecOpenSSLHmacCtxPtr ctx;
-    xmlNodePtr cur;
+    int ret;
 
     xmlSecAssert2(xmlSecOpenSSLHmacCheckId(transform), -1);
     xmlSecAssert2(xmlSecTransformCheckSize(transform, xmlSecOpenSSLHmacSize), -1);
     xmlSecAssert2(node!= NULL, -1);
-    xmlSecAssert2(transformCtx != NULL, -1);
+    UNREFERENCED_PARAMETER(transformCtx);
 
     ctx = xmlSecOpenSSLHmacGetCtx(transform);
     xmlSecAssert2(ctx != NULL, -1);
 
-    cur = xmlSecGetNextElementNode(node->children);
-    if((cur != NULL) && xmlSecCheckNodeName(cur, xmlSecNodeHMACOutputLength, xmlSecDSigNs)) {
-        xmlChar *content;
-
-        content = xmlNodeGetContent(cur);
-        if(content != NULL) {
-            ctx->dgstSize = atoi((char*)content);
-            xmlFree(content);
-        }
-
-        /* Ensure that HMAC length is greater than min specified.
-           Otherwise, an attacker can set this length to 0 or very
-           small value
-        */
-        if((int)ctx->dgstSize < xmlSecOpenSSLHmacGetMinOutputLength()) {
-            xmlSecInvalidNodeContentError(cur, xmlSecTransformGetName(transform),
-                                          "HMAC output length is too small");
-           return(-1);
-        }
-
-        cur = xmlSecGetNextElementNode(cur->next);
-    }
-
-    if(cur != NULL) {
-        xmlSecUnexpectedNodeError(cur, xmlSecTransformGetName(transform));
+    ret = xmlSecTransformHmacReadOutputBitsSize(node, ctx->dgstSize, &ctx->dgstSize);
+    if (ret < 0) {
+        xmlSecInternalError("xmlSecTransformHmacReadOutputBitsSize()",
+            xmlSecTransformGetName(transform));
         return(-1);
     }
+
     return(0);
 }
 
@@ -326,6 +353,85 @@ xmlSecOpenSSLHmacSetKeyReq(xmlSecTransformPtr transform,  xmlSecKeyReqPtr keyReq
     return(0);
 }
 
+#ifndef XMLSEC_OPENSSL_API_300
+static int
+xmlSecOpenSSLHmacSetKeyImpl(xmlSecOpenSSLHmacCtxPtr ctx, const xmlSecByte* key, xmlSecSize keySize) {
+    int keyLen;
+    int ret;
+
+    xmlSecAssert2(ctx != NULL, -1);
+    xmlSecAssert2(ctx->hmacCtx != NULL, -1);
+    xmlSecAssert2(ctx->hmacDgst != NULL, -1);
+    xmlSecAssert2(key != NULL, -1);
+    xmlSecAssert2(keySize > 0, -1);
+
+    XMLSEC_SAFE_CAST_SIZE_TO_INT(keySize, keyLen, return(-1), NULL);
+    ret = HMAC_Init_ex(ctx->hmacCtx, key, keyLen, ctx->hmacDgst, NULL);
+    if(ret != 1) {
+        xmlSecOpenSSLError("HMAC_Init_ex", NULL);
+        return(-1);
+    }
+
+    /* success */
+    return(0);
+}
+
+#else /* XMLSEC_OPENSSL_API_300 */
+
+static int
+xmlSecOpenSSLHmacSetKeyImpl(xmlSecOpenSSLHmacCtxPtr ctx, const xmlSecByte* key, xmlSecSize keySize) {
+    OSSL_PARAM_BLD* param_bld = NULL;
+    OSSL_PARAM* params = NULL;
+    int ret;
+    int res = -1;
+
+    xmlSecAssert2(ctx != NULL, -1);
+    xmlSecAssert2(ctx->evpHmacCtx != NULL, -1);
+    xmlSecAssert2(ctx->evpHmacDgstName != NULL, -1);
+    xmlSecAssert2(key != NULL, -1);
+    xmlSecAssert2(keySize > 0, -1);
+
+    param_bld = OSSL_PARAM_BLD_new();
+    if (param_bld == NULL) {
+        xmlSecOpenSSLError("OSSL_PARAM_BLD_new", NULL);
+        goto done;
+    }
+
+    ret = OSSL_PARAM_BLD_push_utf8_string(param_bld, OSSL_MAC_PARAM_DIGEST,
+        ctx->evpHmacDgstName, strlen(ctx->evpHmacDgstName));
+    if(ret != 1) {
+        xmlSecOpenSSLError("OSSL_PARAM_BLD_push_utf8_string", NULL);
+        goto done;
+    }
+
+    params = OSSL_PARAM_BLD_to_param(param_bld);
+    if (params == NULL) {
+        xmlSecOpenSSLError("OSSL_PARAM_BLD_to_param", NULL);
+        goto done;
+    }
+
+    ret = EVP_MAC_init(ctx->evpHmacCtx, key, keySize, params);
+    if (ret != 1) {
+        xmlSecOpenSSLError("EVP_MAC_init", NULL);
+        goto done;
+    }
+
+    /* success */
+    res = 0;
+
+done:
+    if(params != NULL) {
+        OSSL_PARAM_free(params);
+    }
+    if(param_bld != NULL) {
+        OSSL_PARAM_BLD_free(param_bld);
+    }
+    return(res);
+
+}
+
+#endif /* XMLSEC_OPENSSL_API_300 */
+
 static int
 xmlSecOpenSSLHmacSetKey(xmlSecTransformPtr transform, xmlSecKeyPtr key) {
     xmlSecOpenSSLHmacCtxPtr ctx;
@@ -340,8 +446,6 @@ xmlSecOpenSSLHmacSetKey(xmlSecTransformPtr transform, xmlSecKeyPtr key) {
 
     ctx = xmlSecOpenSSLHmacGetCtx(transform);
     xmlSecAssert2(ctx != NULL, -1);
-    xmlSecAssert2(ctx->hmacCtx != NULL, -1);
-    xmlSecAssert2(ctx->hmacDgst != NULL, -1);
     xmlSecAssert2(ctx->ctxInitialized == 0, -1);
 
     value = xmlSecKeyGetValue(key);
@@ -352,22 +456,17 @@ xmlSecOpenSSLHmacSetKey(xmlSecTransformPtr transform, xmlSecKeyPtr key) {
 
     if(xmlSecBufferGetSize(buffer) == 0) {
         xmlSecInvalidZeroKeyDataSizeError(xmlSecTransformGetName(transform));
-        return(-1);
+       return(-1);
     }
-
     xmlSecAssert2(xmlSecBufferGetData(buffer) != NULL, -1);
 
-    ret = HMAC_Init_ex(ctx->hmacCtx,
-                xmlSecBufferGetData(buffer),
-                xmlSecBufferGetSize(buffer),
-                ctx->hmacDgst,
-                NULL);
-    if(ret != 1) {
-        xmlSecOpenSSLError("HMAC_Init_ex",
-                           xmlSecTransformGetName(transform));
-        return(-1);
+    ret = xmlSecOpenSSLHmacSetKeyImpl(ctx, xmlSecBufferGetData(buffer), xmlSecBufferGetSize(buffer));
+    if(ret < 0) {
+        xmlSecInternalError("xmlSecOpenSSLHmacSetKeyImpl", xmlSecTransformGetName(transform));
+       return(-1);
     }
 
+    /* success */
     ctx->ctxInitialized = 1;
     return(0);
 }
@@ -375,7 +474,7 @@ xmlSecOpenSSLHmacSetKey(xmlSecTransformPtr transform, xmlSecKeyPtr key) {
 static int
 xmlSecOpenSSLHmacVerify(xmlSecTransformPtr transform,
                         const xmlSecByte* data, xmlSecSize dataSize,
-                        xmlSecTransformCtxPtr transformCtx) {
+                        xmlSecTransformCtxPtr transformCtx ATTRIBUTE_UNUSED) {
     static xmlSecByte last_byte_masks[] =
                 { 0xFF, 0x80, 0xC0, 0xE0, 0xF0, 0xF8, 0xFC, 0xFE };
 
@@ -387,7 +486,7 @@ xmlSecOpenSSLHmacVerify(xmlSecTransformPtr transform,
     xmlSecAssert2(transform->operation == xmlSecTransformOperationVerify, -1);
     xmlSecAssert2(transform->status == xmlSecTransformStatusFinished, -1);
     xmlSecAssert2(data != NULL, -1);
-    xmlSecAssert2(transformCtx != NULL, -1);
+    UNREFERENCED_PARAMETER(transformCtx);
 
     ctx = xmlSecOpenSSLHmacGetCtx(transform);
     xmlSecAssert2(ctx != NULL, -1);
@@ -443,7 +542,6 @@ xmlSecOpenSSLHmacExecute(xmlSecTransformPtr transform, int last, xmlSecTransform
     ctx = xmlSecOpenSSLHmacGetCtx(transform);
     xmlSecAssert2(ctx != NULL, -1);
     xmlSecAssert2(ctx->ctxInitialized != 0, -1);
-    xmlSecAssert2(ctx->hmacCtx != NULL, -1);
 
     if(transform->status == xmlSecTransformStatusNone) {
         /* we should be already initialized when we set key */
@@ -455,42 +553,71 @@ xmlSecOpenSSLHmacExecute(xmlSecTransformPtr transform, int last, xmlSecTransform
 
         inSize = xmlSecBufferGetSize(in);
         if(inSize > 0) {
+#ifndef XMLSEC_OPENSSL_API_300
+            xmlSecAssert2(ctx->hmacCtx != NULL, -1);
+
             ret = HMAC_Update(ctx->hmacCtx, xmlSecBufferGetData(in), inSize);
             if(ret != 1) {
                 xmlSecOpenSSLError("HMAC_Update",
                                    xmlSecTransformGetName(transform));
                 return(-1);
             }
+#else /* XMLSEC_OPENSSL_API_300 */
+            xmlSecAssert2(ctx->evpHmacCtx != NULL, -1);
+
+            ret = EVP_MAC_update(ctx->evpHmacCtx, xmlSecBufferGetData(in), inSize);
+            if(ret != 1) {
+                xmlSecOpenSSLError("EVP_MAC_update",
+                                   xmlSecTransformGetName(transform));
+                return(-1);
+            }
+#endif /* XMLSEC_OPENSSL_API_300 */
 
             ret = xmlSecBufferRemoveHead(in, inSize);
             if(ret < 0) {
                 xmlSecInternalError2("xmlSecBufferRemoveHead",
                                      xmlSecTransformGetName(transform),
-                                     "size=%d", inSize);
+                                     "size=" XMLSEC_SIZE_FMT, inSize);
                 return(-1);
             }
         }
 
         if(last) {
-            unsigned int dgstSize = 0;
+            xmlSecSize dgstSize;
 
-            ret = HMAC_Final(ctx->hmacCtx, ctx->dgst, &dgstSize);
+#ifndef XMLSEC_OPENSSL_API_300
+            unsigned int dgstLen = 0;
+
+            xmlSecAssert2(ctx->hmacCtx != NULL, -1);
+            ret = HMAC_Final(ctx->hmacCtx, ctx->dgst, &dgstLen);
             if(ret != 1) {
-                xmlSecOpenSSLError("HMAC_Final",
-                                   xmlSecTransformGetName(transform));
+                xmlSecOpenSSLError("HMAC_Final", xmlSecTransformGetName(transform));
                 return(-1);
             }
+            XMLSEC_SAFE_CAST_UINT_TO_SIZE(dgstLen, dgstSize, return(-1), xmlSecTransformGetName(transform));
+#else /* XMLSEC_OPENSSL_API_300 */
+            size_t dgstSizeT = 0;
+
+            xmlSecAssert2(ctx->evpHmacCtx != NULL, -1);
+            ret = EVP_MAC_final(ctx->evpHmacCtx, ctx->dgst, &dgstSizeT, sizeof(ctx->dgst));
+            if(ret != 1) {
+                xmlSecOpenSSLError("EVP_MAC_final", xmlSecTransformGetName(transform));
+                return(-1);
+            }
+            XMLSEC_SAFE_CAST_SIZE_T_TO_SIZE(dgstSizeT, dgstSize, return(-1), xmlSecTransformGetName(transform));
+#endif /* XMLSEC_OPENSSL_API_300 */
             xmlSecAssert2(dgstSize > 0, -1);
 
             /* check/set the result digest size */
             if(ctx->dgstSize == 0) {
-                ctx->dgstSize = XMLSEC_SIZE_BAD_CAST(dgstSize * 8); /* no dgst size specified, use all we have */
-            } else if(ctx->dgstSize <= XMLSEC_SIZE_BAD_CAST(8 * dgstSize)) {
-                dgstSize = ((ctx->dgstSize + 7) / 8); /* we need to truncate result digest */
+                ctx->dgstSize = dgstSize * 8; /* no dgst size specified, use all we have */
+            } else if(ctx->dgstSize <= 8 * dgstSize) {
+                xmlSecSize adjustedDigestSize = ((ctx->dgstSize + 7) / 8); /* we need to truncate result digest */
+                XMLSEC_SAFE_CAST_SIZE_TO_UINT(adjustedDigestSize, dgstSize, return(-1), xmlSecTransformGetName(transform));
             } else {
                 xmlSecInvalidSizeLessThanError("HMAC digest (bits)",
-                                        8 * dgstSize, ctx->dgstSize,
-                                        xmlSecTransformGetName(transform));
+                    8 * dgstSize, ctx->dgstSize,
+                    xmlSecTransformGetName(transform));
                 return(-1);
             }
 
@@ -499,8 +626,8 @@ xmlSecOpenSSLHmacExecute(xmlSecTransformPtr transform, int last, xmlSecTransform
                 ret = xmlSecBufferAppend(out, ctx->dgst, dgstSize);
                 if(ret < 0) {
                     xmlSecInternalError2("xmlSecBufferAppend",
-                                         xmlSecTransformGetName(transform),
-                                         "size=%d", dgstSize);
+                        xmlSecTransformGetName(transform),
+                        "size=" XMLSEC_SIZE_FMT, dgstSize);
                     return(-1);
                 }
             }
